/*
	ann.c 
	
	Released under GPL - see the 'COPYING' file   
	
	Copyright (C) 2006 Timo Lassmann <timolassmann@gmail.com>
	
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    
	Please send bug reports, comments etc. to:
	timolassmann@gmail.com
*/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>
#include "msa_viewer.h"

static int color_order[144] = {0,70,135,18,24,27,30,34,51,66,73,86,132,133,134,135,127,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50};

int get_full_names(struct alignment* aln,int full_name)
{
	int i;
	int max = -1;
	for (i = 0; i < aln->numseq;i++){
		if( aln->si[i]->namel > max){
			max = aln->si[i]->namel;
		}
	}
	
	if(full_name == -1){
		full_name = 15;
	}
	
	if(max < full_name){
		full_name = max+1;
	//	for (j = 0; j< aln->numseq;j++){
	//		 aln->si[j]->name[max] = 0;
	//	}
	}
	if(full_name < 7) full_name = 7;
	return full_name;
}

struct alignment* annotation_select(struct alignment* aln,char* ann)
{
	if (byg_start(ann,"conservationCONSERVATIONblosumBLOSUM") != -1){
		aln = color_by_average_blosum62_score(aln);
		fprintf(stderr,"CONSERVATION....\n");
	}else if(byg_start(ann,"erikERIK") != -1){
		aln = erik_annotation(aln);
	}else if(byg_start(ann,"tobyTOBY") != -1){
		aln = toby_annotation(aln);
	}else if(byg_start(ann,"hydro7") != -1){
		aln =  hydrophobicity_annotation(aln,7);
	}else if(byg_start(ann,"hydro21") != -1){
		aln =  hydrophobicity_annotation(aln,21);
	}else if(byg_start(ann,"karolinskaKarolinskaKIki") != -1){
		aln = color_by_average_blosum62_score(aln);
		ki_color_transform(aln);
	}else if(byg_start(ann,"CMSAcmsa") != -1){
		aln = cmsa_annotation(aln);
	}else{
		aln =  blank_annotation(aln);
	}
	return aln;	
}

/*
struct alignment* column_annotation(struct alignment* aln,int num_type);
struct alignment* feature_annotation(struct alignment* aln,char* type);
struct alignment* cmsa_annotation(struct alignment* aln);
struct alignment* toby_annotation(struct alignment* aln);
struct alignment* erik_annotation(struct alignment* aln);
struct alignment* blank_annotation(struct alignment* aln);
struct alignment*  color_by_average_blosum62_score(struct alignment* aln);
void ki_color_transform(struct alignment* aln);*/


void ki_color_transform(struct alignment* aln)
{
	int i,j, choice;
	for (i = 0;i < aln->numseq;i++){
		for (j = 0;j < aln->aln_len;j++){
			choice = aln->ann[i][j];
			if(choice == 3){//aqua
				aln->ann[i][j] = 144;
			}else if(choice == 40){//deepSkyblue
				aln->ann[i][j] = 146;
			}else if(choice == 69){//lightgrey
				aln->ann[i][j] = 147;
			}
		}
	}
}

struct alignment* hydrophobicity_annotation(struct alignment* aln,int window)
{

	//Kyte-Doolittle scale:
	/*Alanine             1.8          A 
	Cysteine            2.5      C
	Aspartic acid      -3.5     D
	Glutamic acid      -3.5    E
	Phenylalanine       2.8     F     
	Glycine            -0.4      G
	Histidine          -3.2        H 
	Isoleucine          4.5      I
	Lysine             -3.9     K
	Leucine             3.8       I
	Methionine          1.9  M          
	Asparagine         -3.5      N
	Proline            -1.6      P
	Glutamine          -3.5     Q 
	Arginine           -4.5          R
	Serine             -0.8       S
	Threonine          -0.7   T     
	Valine              4.2  V
	Tryptophan         -0.9    W    	
	Tyrosine           -1.3     Y*/
	//int aacode[26] = {0,1,2,3,4,5,6,7,8,-1,9,10,11,12,-1,13,14,15,16,17,-1,18,19,20,21,22};
	//				   ABCDEFGHIJKLMNOPQRSTUVWXYZ
	//				   A- CDEFGHI-KLMN-  PQRST-  VW-Y-
	
	int i,j,c;
	double kd[26] = {1.8,0,2.5,-3.5,-3.5,2.8,-0.4,-3.2,4.5,0,-3.9,3.8,1.9,-3.5,0,-1.6,-3.5,-4.5,-0.8,-0.7,0,4.2,-0.9,0,-1.3,0};
	double* hydrophobicity = 0;
	char* seq = 0;
	double tmp = 0.0;
	
	int len;
	
	
	if (window == 7){
		aln->ann_key[4] = 1;	
	}else{
		aln->ann_key[5] = 1;
	}
	hydrophobicity = tmalloc(sizeof(double)*aln->aln_len);
	
	seq =  tmalloc(sizeof(char)*aln->aln_len);
	
	aln->ann = tmalloc(sizeof(int*)* aln->numseq);
	for (i = aln->numseq;i--;){
		aln->ann[i] = tmalloc(sizeof(int)*aln->aln_len);
		for (j = aln->aln_len;j--;){
			aln->ann[i][j] = 0;
		}
	}
	
	
	
	for (i = aln->numseq;i--;){
		if(aln->si[i]->sl > window){
		len = 0;
		for (j = 0;j < aln->aln_len;j++){
			hydrophobicity[j] = 0;
			if(isalpha(aln->si[i]->s[j])){
				seq[len] = aln->si[i]->s[j];
				len++;
			}
		}
			
		
		
		for (j = 0;j <= len -window;j++){
			tmp = 0;
			for ( c = 0; c < window;c++){
				tmp += kd[toupper(seq[j+c])-65];
			}
			tmp /= window;
			hydrophobicity[j+window/2] = tmp;
			
		}
		
		for (j = 0; j < window/2;j++){
			hydrophobicity[j] = -99;
		}
		for (j = len- window/2; j < len;j++){
			hydrophobicity[j] = -99;
		}
		
		c = 0;
		for (j = 0;j < aln->aln_len;j++){
			if(isalpha(aln->si[i]->s[j])){
				switch((int) hydrophobicity[c]){
					case -4:
						aln->ann[i][j] = 22; //Darkblue
						break;
					case -3:
						aln->ann[i][j] = 10; //Blue
						break;
					case -2:
						aln->ann[i][j] = 42; //DodgerBlue
						break;
					case -1:
						aln->ann[i][j] = 74; //LightSkyBlue
						break;
					case 0:
						aln->ann[i][j] = 0;
						break;
					case 1:
						aln->ann[i][j] = 71;//{"LightPink","#FFB6C1",71},
						break;
					case 2:
						aln->ann[i][j] = 135;//{"Tomato","#FF6347",135},
						break;
					case 3:
						aln->ann[i][j] = 20;//{"Crimson","#DC143C",20},
						break;
					case 4:
						aln->ann[i][j] = 116;//{"Red","#FF0000",116},hydrophobicity[c];
						break;
					case -99:
						aln->ann[i][j] = 27;//Khaki 
						break;
						
					default:
	//					fprintf(stderr,"%d\n",(int) hydrophobicity[c]);
						aln->ann[i][j] = 0;
						break;
					
				}
				
				c++;
			}
		}
		}
	}
	
	for (i = 0; i < aln->numseq;i++){
		for (j = 0; j < aln->aln_len;j++){
			if(!isalpha(aln->si[i]->s[j])){
				aln->ann[i][j] = 27;//Khaki 
			}else{
				break;
			}
		}
		
		for (j = aln->aln_len;j--;){
			if(!isalpha(aln->si[i]->s[j])){
				aln->ann[i][j] = 27;//Khaki 
			}else{
				break;
			}
		}
	}
	
	free(hydrophobicity);
	free(seq);
	return aln;
}

struct alignment* column_annotation(struct alignment* aln,int num_type)
{
	int i,j,c;
	int c1,c2;
	double* col = 0;
	
	aln->ann = tmalloc(sizeof(int*)* aln->numseq);
	for (i = aln->numseq;i--;){
		aln->ann[i] = tmalloc(sizeof(int)*aln->aln_len);
		for (j = aln->aln_len;j--;){
			aln->ann[i][j] = 0;
		}
	}
	if(byg_start("coreblock",aln->ca[num_type]->name)!= -1){
		col = aln->ca[num_type]->doublec;
		for(i = aln->aln_len;i--;){
			switch((int)col[i]){
				case -1:
					c = 0;
					break;
				case 0:
					c = 121;
					break;
				case 1:
					c = 116;//red....
					break;
				default:
					c = 0;
					break;
			}
			for (j = aln->numseq;j--;){
				if(aln->si[j]->group == aln->ca[num_type]->owner ){
					aln->ann[j][i] = c;
				}
			}
		}
	}
	if(byg_start("RSS",aln->ca[num_type]->name)!= -1){
		col = aln->ca[num_type]->doublec;
		for(i = aln->aln_len;i--;){
			switch((int)col[i]){
				case 0:
					c1 = 0;
					c2 = 0;
					break;
				case 1:
					c1 = 66;//56; //HotPink LightCoral 
					c2 = 77;//74;// LightSkyBlue  LightSteelBlue 
					break;
				default:
					c1 = 0;
					c2 = 0;
					break;
			}
			for (j = aln->numseq;j--;){
				if(isalpha(aln->si[j]->s[i])){
				if(aln->si[j]->group == 0){
					aln->ann[j][i] = c1;
				}else{
					aln->ann[j][i] = c2;
				}
				}
				
			}
		}
	}
	if(byg_start("CSS",aln->ca[num_type]->name)!= -1){
		col = aln->ca[num_type]->doublec;
		for(i = aln->aln_len;i--;){
			switch((int)col[i]){
				case 0:
					c1 = 0;
					c2 = 0;
					break;
				case 1:
					c1 = 142;//60;//142;//Yellow Khaki 
					c2 = 70; //Lime//LightGreen 
					break;
				default:
					c1 = 0;
					c2 = 0;
					break;
			}
			for (j = aln->numseq;j--;){
				if(isalpha(aln->si[j]->s[i])){
				if(aln->si[j]->group == 0){
					aln->ann[j][i] = c1;
				}else{
					aln->ann[j][i] = c2;
				}
				}
				
			}
		}
	}
	if(byg_start("RSS/CSS",aln->ca[num_type]->name)!= -1){
		col = aln->ca[num_type]->doublec;
		for(i = aln->aln_len;i--;){
			switch((int)col[i]){
				case 0:
					c1 = 0;
					c2 = 0;
					break;
				case 1: //css
					c1 = 142;//60;//142;//Yellow Khaki 
					c2 = 70; //Lime//LightGreen 
					break;
				case 2: //rss
					c1 = 66;//56; //HotPink LightCoral 
					c2 = 77;//74;// LightSkyBlue  LightSteelBlue 
					break;
				case 3:
					c1 = 72;//103; //Orange OrangeRed GreenYellow 
					c2 = 54; //Yellow Green LightSalmon 
					break;
				default:
					c1 = 0;
					c2 = 0;
					break;
			}
			for (j = aln->numseq;j--;){
				if(isalpha(aln->si[j]->s[i])){
				if(aln->si[j]->group == 0){
					aln->ann[j][i] = c1;
				}else{
					aln->ann[j][i] = c2;
				}
				}
				
			}
		}
	}
	
	return aln;
}


struct alignment* blank_annotation(struct alignment* aln)
{
	int i,j;
	
	
	aln->ann_key[0] =1;
	
	aln->ann = tmalloc(sizeof(int*)* aln->numseq);
	for (i = aln->numseq;i--;){
		aln->ann[i] = tmalloc(sizeof(int)*aln->aln_len);
		for (j = aln->aln_len;j--;){
			aln->ann[i][j] = 0;
		}
	}
	return aln;
}

struct alignment* cmsa_annotation(struct alignment* aln)
{
	struct feature* n = 0;
	int i,j;
	
	aln->ann = tmalloc(sizeof(int*)* aln->numseq);
	for (i = aln->numseq;i--;){
		aln->ann[i] = tmalloc(sizeof(int)*aln->aln_len);
		for (j = aln->aln_len;j--;){
			aln->ann[i][j] = 0;
		}
	}

	for (i = 0;i < aln->numseq;i++){
		n = aln->si[i]->ft;
		while(n){
			//if(strlen(n->type) == strlen(type)){
			if(check_identity(n->type,"CMSA")!=-1){
				switch(atoi(n->note)){
					case 1:
						n->color = 116;//#FF0000
						break;
					case 2:
						n->color = 103;//#FF4500
						break;
					case 3:
						n->color = 102;//#FFA500
						break;
					case 4:
						n->color = 50;//#FFD700
						break;
					case 5:
						n->color = 80;//#32CD32
						break;
					case 6:
						n->color = 73;//#20B2AA
						break;
					case 7:
						n->color = 40;//#00BFFF
						break;
					case 8:
						n->color = 10;//#0000FF
						break;
					case 9:
						n->color = 0;
						break;
					default:
						n->color = 0;
						break;
				}

				//fprintf(stderr,"%s	: %s	%d-%d	%d\n",n->type,n->note,n->rel_start, n->rel_end,n->color);
				if(n->rel_start >= 0 && n->rel_end >= 0){
					for(j = n->rel_start; j <= n->rel_end;j++){
						aln->ann[i][j] = n->color;
					}				
				}
			}
		//	}
			n = n->next;
		}
	}

	return aln;
}

struct alignment* feature_annotation(struct alignment* aln,char* type)
{
	struct feature* n = 0;
	int i,j;
	
	aln->ann = tmalloc(sizeof(int*)* aln->numseq);
	for (i = aln->numseq;i--;){
		aln->ann[i] = tmalloc(sizeof(int)*aln->aln_len);
		for (j = aln->aln_len;j--;){
			aln->ann[i][j] = 0;
		}
	}
	/*if(check_identity("CMSA",type)!=-1){
		for (i = 0;i < aln->numseq;i++){
			n = aln->si[i]->ft;
			while(n){
				//if(strlen(n->type) == strlen(type)){
				//if(check_identity(n->type,type)!=-1){
					if(n->rel_start >= 0 && n->rel_end >= 0){
						for(j = n->rel_start; j <= n->rel_end;j++){
							aln->ann[i][j] = atoi(n->note)+40;
						}				
					}
				//}
				n = n->next;
			}
		}
	}else{*/
	
	for (i = 0;i < aln->numseq;i++){
		n = aln->si[i]->ft;
		while(n){
			//if(strlen(n->type) == strlen(type)){
			if(check_identity(n->type,type)!=-1){
				
				//fprintf(stderr,"%s	%s: %s	%d-%d	%d --- %d\n",n->type,type,n->note,n->rel_start, n->rel_end,n->color,color_order[n->color]);

				n->color = color_order[n->color];
				
				if(n->rel_start >= 0 && n->rel_end >= 0){
					for(j = n->rel_start; j <= n->rel_end;j++){
						aln->ann[i][j] = n->color;
					}				
				}
			}
		//	}
			n = n->next;
		}
	}
	//}
	//fprintf(stderr,"\n\n\n");
/*	if(byg_start("PFAM-B",type)!=-1){
		for (i = aln->numseq;i--;){
			for (j = aln->aln_len;j--;){
				fprintf(stderr,"%d",aln->ann[i][j]);
			}
			fprintf(stderr,"\n");
		}
		fprintf(stderr,"\n");
		exit(0);
	}*/
	return aln;
}



struct alignment* erik_annotation(struct alignment* aln)
{
	int i,j;
	
	
	
	aln->ann_key[3] = 1;

	aln->ann = tmalloc(sizeof(int*)* aln->numseq);
	for (i = aln->numseq;i--;){
		aln->ann[i] = tmalloc(sizeof(int)*aln->aln_len);
	}
		
	for(i = aln->numseq;i--;){
		for (j = aln->aln_len;j--;){
	//		switch (aln->s[i][j]){
			switch (aln->si[i]->s[j]){
				case 'A':
				case 'F':
				case 'I':
				case 'L':
				case 'M':
				case 'V':
				case 'W':
				case 'Y':		
					aln->ann[i][j] = 142;//6;//((int)'y'<<8)+ (int)'e';
					break;
				case 'B':
					aln->ann[i][j] = 0;
				break;
				case 'C':
					aln->ann[i][j] = 40;//5;//((int)'b'<<8)+ (int)'l';
				break;
				case 'G':
				case 'P':
					aln->ann[i][j] = 3;//10;//((int)'c'<<8)+ (int)'y';
				break;
				case 'H':
				case 'K':
				case 'R':
					aln->ann[i][j] = 79;//8;//((int)'l'<<8)+ (int)'g';
				break;
				case 'D':
				case 'E':
				case 'N':
				case 'Q':
				case 'S':
				case 'T':
					aln->ann[i][j] = 66;//1;//((int)'l'<<8)+ (int)'r';
				break;
				
				case 'Z':
					aln->ann[i][j] = 0;
				break;
				default:
					aln->ann[i][j] = 0;
				break;
			}
		}
	}
	return aln;
}






struct alignment* toby_annotation(struct alignment* aln)
{
	int i,j;
		
	aln->ann_key[2] = 1;

	aln->ann = tmalloc(sizeof(int*)* aln->numseq);
	for (i = aln->numseq;i--;){
		aln->ann[i] = tmalloc(sizeof(int)*aln->aln_len);
	}
	for(i = aln->numseq;i--;){
		for (j = aln->aln_len;j--;){
	//		switch (aln->s[i][j]){
			switch (aln->si[i]->s[j]){
				case 'B':
				case 'Z':
				default:
					aln->ann[i][j] = 0;
				//		fprintf(stderr,"%c",p[i]);
					break;
				case 'A':
				case 'C':
				case 'F':
				case 'I':
				case 'L':
				case 'M':
				case 'V':
				case 'W':
					aln->ann[i][j] = 40;//5;
			//			fprintf(stderr,"<bl>%c</bl>",p[i]);
				break;
				
				case 'D':
				case 'E':
					aln->ann[i][j] = 86;//3;
				//		fprintf(stderr,"<pu>%c</pu>",p[i]);
					break;
				case 'G':
					aln->ann[i][j] = 102;//4;
				//		fprintf(stderr,"<or>%c</or>",p[i]);
					break;
				case 'H':
					aln->ann[i][j] = 74;//2;
				//		fprintf(stderr,"<lr>%c</lr>",p[i]);
					break;
				case 'K':
				case 'R':
					aln->ann[i][j] = 116;//9;
				//		fprintf(stderr,"<re>%c</re>",p[i]);
					break;
					
				case 'N':
				case 'Q':
				case 'S':
				case 'T':
					aln->ann[i][j] = 79;//8;
				//		fprintf(stderr,"<lg>%c</lg>",p[i]);
					break;
				case 'P':
					aln->ann[i][j] = 142;//6;
				//		fprintf(stderr,"<ye>%c</ye>",p[i]);
					break;
				case 'Y':
					aln->ann[i][j] = 74;//2;
				//		fprintf(stderr,"<lb>%c</lb>",p[i]);
					break;
			}
		}
	}
	return aln;
}


struct alignment* default_dna_annotation(struct alignment* aln)
{
	int i,j;
	
	
	
	aln->ann_key[6] = 1;

	aln->ann = tmalloc(sizeof(int*)* aln->numseq);
	for (i = aln->numseq;i--;){
		aln->ann[i] = tmalloc(sizeof(int)*aln->aln_len);
	}
	for(i = aln->numseq;i--;){
		for (j = aln->aln_len;j--;){
	//		switch (aln->s[i][j]){
			switch (aln->si[i]->s[j]){
				case 'A':
					//LightSlateblue
					aln->ann[i][j] = 75;
					break;
				case 'G':
					//Lightcoral
					aln->ann[i][j] = 66;
					break;
				case 'I':
					aln->ann[i][j] = 21;
					break;
				case 'C':
					aln->ann[i][j] = 121;
					break;
				case 'T':
					//LightGreen
					aln->ann[i][j] = 70;
					break;
				case 'U':
					//LightPink
					//aln->ann[i][j] = 71;
					//Grey
					aln->ann[i][j] =69;
					break;
					
				default:
					aln->ann[i][j] = 0;
					break;
			}
		}
	}
	return aln;
}

struct alignment* strong_dna_annotation(struct alignment* aln)
{
	int i,j;
	
	
	
	aln->ann_key[7] = 1;

	aln->ann = tmalloc(sizeof(int*)* aln->numseq);
	for (i = aln->numseq;i--;){
		aln->ann[i] = tmalloc(sizeof(int)*aln->aln_len);
	}
	for(i = aln->numseq;i--;){
		for (j = aln->aln_len;j--;){
	//		switch (aln->s[i][j]){
			switch (aln->si[i]->s[j]){
				case 'A':
					//Lime Green
					aln->ann[i][j] = 80;
					break;
				case 'G':
					//Orange
					aln->ann[i][j] = 102;
					break;
				case 'I':
					aln->ann[i][j] = 0;
					break;
				case 'C':
					//blue
					aln->ann[i][j] = 118;
					break;
				case 'T':
					//Red 
					aln->ann[i][j] = 116;
					break;
				case 'U':
					aln->ann[i][j] =0;
					break;
					
				default:
					aln->ann[i][j] = 0;
					break;
			}
		}
	}
	return aln;
}

struct alignment* simple_conservation_annotation(struct alignment* aln)
{
	int i,j;
	double freq[26];
	double max;
	int col = 0;
	
	
	
	aln->ann_key[8] = 1;

	aln->ann = tmalloc(sizeof(int*)* aln->numseq);
	for (i = aln->numseq;i--;){
		aln->ann[i] = tmalloc(sizeof(int)*aln->aln_len);
	}

	for (i = aln->aln_len;i--;){
		for(j = 0; j < 26;j++){
			freq[j] = 0.0;
		}
		for(j = 0; j < aln->numseq;j++){
			if(isalpha((int)aln->si[j]->s[i])){
				freq[(int)(aln->si[j]->s[i]-65)]++;
			}
		}
		max = 0.0;
		for(j = 0; j < 26;j++){
			freq[j] /= aln->numseq;
			if(freq[j] > max){
				max = freq[j];
			}
		}
		if(max > 0.9){
			//Blue
			col = 10;
		}else if(max > 0.8){
			//MediumBlue 
			col = 85;
		}else if(max > 0.7){
			//MediumSlateBlue 
			col = 89;
		}else if(max > 0.6){
			//LightSlateBlue 
			col = 75;
		}else if(max > 0.5){
			//SkyBlue 
			col = 126;
		}else if(max > 0.4){
			//LightSkyBlue 
			col = 74;
		}else if(max > 0.3){
			//lightblue
			col = 65;
		}else if(max > 0.2){
			//lightcyan
			col = 67;
		}else {
			col = 0;
		}
		for(j = 0; j < aln->numseq;j++){
			if(isalpha((int)aln->si[j]->s[i])){
				aln->ann[j][i] = col;
			}else{
				aln->ann[j][i] = 0;
			}
		}
	}
	return aln;
}



struct alignment*  color_by_average_blosum62_score(struct alignment* aln)
{
	short blosum62mt[]={
  4,
 -2,  4,
  0, -3,  9,
 -2,  4, -3,  6,
 -1,  1, -4,  2,  5,
 -2, -3, -2, -3, -3,  6,
  0, -1, -3, -1, -2, -3,  6,
 -2,  0, -3, -1,  0, -1, -2,  8,
 -1, -3, -1, -3, -3,  0, -4, -3,  4,
 -1,  0, -3, -1,  1, -3, -2, -1, -3,  5,
 -1, -4, -1, -4, -3,  0, -4, -3,  2, -2,  4,
 -1, -3, -1, -3, -2,  0, -3, -2,  1, -1,  2,  5,
 -2,  3, -3,  1,  0, -3,  0,  1, -3,  0, -3, -2,  6,
 -1, -2, -3, -1, -1, -4, -2, -2, -3, -1, -3, -2, -2,  7,
 -1,  0, -3,  0,  2, -3, -2,  0, -3,  1, -2,  0,  0, -1,  5,
 -1, -1, -3, -2,  0, -3, -2,  0, -3,  2, -2, -1,  0, -2,  1,  5,
  1,  0, -1,  0,  0, -2,  0, -1, -2,  0, -2, -1,  1, -1,  0, -1,  4,
  0, -1, -1, -1, -1, -2, -2, -2, -1, -1, -1, -1,  0, -1, -1, -1,  1,  5,
  0, -3, -1, -3, -2, -1, -3, -3,  3, -2,  1,  1, -3, -2, -2, -3, -2,  0,  4,
 -3, -4, -2, -4, -3,  1, -2, -2, -3, -3, -2, -1, -4, -4, -2, -3, -3, -2, -3, 11,
  0, -1, -2, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -2, -1, -1,  0,  0, -1, -2, -1,
 -2, -3, -2, -3, -2,  3, -3,  2, -1, -2, -1, -1, -2, -3, -1, -2, -2, -2, -1,  2, -1,  7,
 -1,  1, -3,  1,  4, -3, -2,  0, -3,  1, -3, -1,  0, -1,  3,  0,  0, -1, -2, -3, -1, -2,  4};
 
 	int m_pos,i,j,c;
 	int subm[32][32];
 	int aacode[26] = {0,1,2,3,4,5,6,7,8,-1,9,10,11,12,-1,13,14,15,16,17,-1,18,19,20,21,22};

	m_pos = 0;
	for (i = 0;i < 23;i++){
		for (j = 0;j <= i;j++){
			if (i == j){
				subm[i][j] = blosum62mt[m_pos];
			}else{
				subm[i][j] = blosum62mt[m_pos];
				subm[j][i] = blosum62mt[m_pos];
			}
			m_pos++;
		}
	}
	
	aln->ann_key[1] = 1;

	aln->ann = tmalloc(sizeof(int*)*aln->numseq);
	for (i = aln->numseq;i--;){
		aln->ann[i] = tmalloc(sizeof(int)*aln->aln_len);
		for (j = 0;j < aln->aln_len;j++){
			aln->ann[i][j] = 0;
		}
	}
 	
 	
 	
	double average_score[23];
	int freq[23];

	for ( i =0 ;i < aln->aln_len;i++){
		for (j = 0; j < 23;j++){
			average_score[j] = 0;
			freq[j] = 0;
		}
		for (j = 0; j < aln->numseq;j++){
			if(isalpha((int)aln->si[j]->s[i])){
				freq[aacode[(int)aln->si[j]->s[i]-65]]++;
			}
		}
		
		for (j = 0; j < 23;j++){
			for (c = 0; c < 23;c++){
				if(j == c){
					average_score[j] +=  freq[j]*(freq[j]-1) *subm[j][j];
				}else{
					average_score[j] +=  freq[j]*freq[c] *subm[j][c];
				}
			}
			average_score[j] /= ((aln->numseq*(aln->numseq-1)));
			if (average_score[j] > 3.0){
				freq[j] = 9;//color code!!!!!!!!!!!
			}else if(average_score[j] > 1.5){
				freq[j] = 8;//color code!!!!!!!!!!!
			}else if(average_score[j] > 0.5){
				freq[j] = 7;//color code!!!!!!!!!!!
			}else{
				freq[j] = 0;//color code!!!!!!!!!!!
			}
			average_score[j] = 0;
		}
		for (j = 0; j < 23;j++){
			if (freq[j] == 9 && !average_score[j]){
				average_score[j] = 9;
				for (c = 0; c < 23;c++){
					if (subm[j][c] > 0){
						average_score[c] = 3;// 10;//((int)'c'<<8)+ (int)'y';aqua
					}
				}
			}
			
			if (freq[j] == 8 && !average_score[j]){
				average_score[j] = 8;
				for (c = 0; c < 23;c++){
					if (subm[j][c] > 0){
						average_score[c] = 40;//5;//((int)'b'<<8)+ (int)'l';deepSkyblue
					}
				}
			}
			
			if (freq[j] == 7 && !average_score[j]){
				average_score[j] = 7;
				for (c = 0; c < 23;c++){
					if (subm[j][c] > 0){
						average_score[c] = 69;// 11;//((int)'g'<<8)+ (int)'r';LightGrey
					}
				}
			}	
		}
		for (j = 0; j < aln->numseq;j++){
			if(isalpha((int)aln->si[j]->s[i])){
				aln->ann[j][i] = average_score[aacode[aln->si[j]->s[i]-65]];
			}
		}
	}
	return aln;
}
