/*
	circular_svg_tree.c 
	
	Released under GPL - see the 'COPYING' file   
	
	Copyright (C) 2006 Timo Lassmann <timolassmann@gmail.com>
	
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    
	Please send bug reports, comments etc. to:
	timolassmann@gmail.com
*/

#include "msa_viewer.h"
#include "phylo.h"
#include "svg_tree.h"
#include <math.h>


double* c;
double* dx;
double* dy;
double *s;

double i_global;
double k;


double sigmad = 0.0;

void circular_svg_tree(struct tnode* tree)
{
	int j;
	i_global = 0.0;
	k = 0.0;

	s = malloc(sizeof(double)*(numseq*2));
	c = malloc(sizeof(double)*numseq*2);
	dx = malloc(sizeof(double)*numseq*2);
	dy = malloc(sizeof(double)*numseq*2);
	
	for (j = 0; j < numseq ;j++){
		c[j] = 0.0;
		dx[j] = 0.0;
		dy[j] = 0.0;
	}
	
	k = numseq; // k = k +1 if v element of V has degree of 1...
	
	//postorder_traversal - done alreagy size holds number of leaves for each subtree... 
	minx = FLOATINFTY;
	maxx = -FLOATINFTY ;

	miny = FLOATINFTY;
	maxy = -FLOATINFTY;
	circular_postorder_traversal(tree);
	circular_preorder_traversal(tree);
	sigma = sigma/sigmad;
	
	//fprintf(stderr,"%f\n",sigma);
	

	free(s);
	free(c);
	free(dx);
	free(dy);
	
}

void circular_preorder_traversal(struct tnode* v)
{
	int f = 0;
	
	if(v->leaf == -1){
		v->x = dx[v->num];
		v->y = dy[v->num];
	}else{
		v->w = 2*M_PI / numseq;
		//v->tau =  0;
	
		
		v->x = c[v->num]*v->parent->x + dx[v->num];
		v->y = c[v->num]*v->parent->y + dy[v->num];
		
		if(dx[v->num] > 0){
			//v->tau =  atan(dy[v->num]/dx[v->num]);
			v->tau =  atan(v->y/v->x);
		}else{
			//v->tau =  M_PI +  atan(dy[v->num]/dx[v->num]);
			v->tau =   M_PI + atan(v->y/v->x);
		}
		//if(dy[v->num] > 0.0 && dx[v->num] > 0){
		//	v->tau += M_PI *1.5;
		//}
		
		//fprintf(stderr,"%f	%f	%f\n",dx[v->num],dy[v->num],v->tau);
		
		sigma += sqrt(pow(v->parent->x - v->x,2) + pow(v->parent->y - v->y,2));
		sigmad += v->e;
	}

	if(v->x > maxx){
		maxx = v->x;
	}
		
	if(v->x < minx){
		minx = v->x;
	}
	
	if(v->y > maxy){
		maxy = v->y;
	}
		
	if(v->y < miny){
		miny = v->y;
	}

	while(v->links[f]){
		circular_preorder_traversal(v->links[f]);
		f++;
	}

	/*if(v->left){
		circular_preorder_traversal(v->left);
	}
	if(v->right){
		circular_preorder_traversal(v->right);
	}*/
}


void circular_postorder_traversal(struct tnode* v)
{
	double S;
	double t;
	double tx;
	double ty;
	int f = 0;

	while(v->links[f]){
		circular_postorder_traversal(v->links[f]);
		f++;
	}
	
	/*if(v->left){
		circular_postorder_traversal(v->left);
	}
	if(v->right){
		circular_postorder_traversal(v->right);
	}*/
	
	if(v->leaf == 1){ // need to add possibility if root has one degreee... 
		c[v->num] = 0.0;
		dx[v->num] = cos((2*M_PI*i_global)/k);
		dy[v->num] = sin((2*M_PI*i_global)/k);
		
		
		i_global  = i_global  + 1;
	}else{
		S = 0.0;
		if(v->leaf == -1){
			f = 0;
			while(v->links[f]){
				s[v->links[f]->num] = 1/v->links[f]->e;
				S += s[v->links[f]->num];
				f++;
			}
			s[v->num] = 0.0;
			
			//s[v->left->num] = 1/v->left->e;
			//S += s[v->left->num];
			//s[v->right->num] = 1/v->right->e;
			//S += s[v->right->num];
		}else if(v->leaf != 1){
			f = 0;
			while(v->links[f]){
				s[v->links[f]->num] = 1/(v->links[f]->e*(v->deg-1));
			//	fprintf(stderr,"edgeL:%f	%d	%d\n",v->links[f]->e,v->deg-1,v->links[f]->num);
				S += s[v->links[f]->num];
				f++;
			}
			s[v->num] = 1/v->e;
			S += s[v->num];
		
			//s[v->left->num] = 1/(v->left->e*(v->deg-1));
			//S += s[v->left->num];
			//s[v->right->num] = 1/(v->right->e*(v->deg-1));
			//S += s[v->right->num];
			//s[v->num] = 1/v->e;
			//S += s[v->num];
		}else{
			s[v->num] = 1/v->e;
			S += s[v->num];
		}
		//fprintf(stderr,"%d\n",v->num);
		//fprintf(stderr,"%d\n",v->deg);
		//fprintf(stderr,"%f\n",v->e);
		//fprintf(stderr,"%f\n",s[v->num]);
		//fprintf(stderr,"S:%f\n",S);
		
		t = 0.0;
		tx = 0.0;
		ty = 0.0;
		
		f = 0;
		while(v->links[f]){
			t = t + ((s[v->links[f]->num]/S)*c[v->links[f]->num]);
			tx = tx + ((s[v->links[f]->num]/S)*dx[v->links[f]->num]);
			ty = ty + ((s[v->links[f]->num]/S)*dy[v->links[f]->num]);
			//fprintf(stderr,"s[]:%f	c:%f	dx:%f	dy:%f	%d\n",s[v->links[f]->num],c[v->links[f]->num],dx[v->links[f]->num],dy[v->links[f]->num],v->links[f]->num);
			f++;
		}
		//fprintf(stderr,"%f	%f	%f\n",t,tx,ty);
		/*if(v->left){
			t = t + ((s[v->left->num]/S)*c[v->left->num]);
			tx = tx + ((s[v->left->num]/S)*dx[v->left->num]);
			ty = ty + ((s[v->left->num]/S)*dy[v->left->num]);
		}
		if(v->right){
			t = t + ((s[v->right->num]/S)*c[v->right->num]);
			tx = tx + ((s[v->right->num]/S)*dx[v->right->num]);
			ty = ty + ((s[v->right->num]/S)*dy[v->right->num]);
		}*/
		
		if(v->leaf != -1){
			c[v->num] = s[v->num]/(S*(1-t));
		}
		dx[v->num] = tx/(1-t); 
		dy[v->num] = ty/(1-t);
		//fprintf(stderr,"%f	%f	%f\n",dx[v->num] ,dy[v->num],c[v->num] );
	}
	if(isnan(dx[v->num])){
		exit(0);
	}
	
}



