/*
	msa_viewer.h 
	
	Released under GPL - see the 'COPYING' file   
	
	Copyright (C) 2006 Timo Lassmann <timolassmann@gmail.com>
	
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    
	Please send bug reports, comments etc. to:
	timolassmann@gmail.com
*/

#include <stdio.h>
#include <stdlib.h>


struct sequence_information{
	struct feature* ft; 
	char* s;
	char* name;
	char* accession;
	char* nid;
	char* ec;
	int sl;
	int namel;
	int accessionl;
	int nidl;
	int ecl;
	int group;
	int abs_length;
	int weight;
};

struct column_ann{
	char* name;
	double* doublec;
	int owner;
};

struct feature{
	struct feature *next;
	char* type;
	char* note;
	int start;
	int rel_start;
	int end;
	int rel_end;
	int color;
};

struct utype_ufeat{
	struct feature *t;
	struct feature *f;
};

struct color{
	char* name;
	char* hex_code;
	int num;
};

struct alignment{
	struct column_ann** ca; 
	struct sequence_information** si;
	int** ann;
	int* ann_key;
	int numseq;
	int aln_len;
	char** types;
	int numtypes;
	int numcolumn;
};

struct parameters{
	char* outfile;
	char* input;
	char* reformat;
	char* kalign_parameters;
	char* crosslink;
	char* ann;
	char* tree_layout;
	int tree;
	int remove_allgap_columns;
	int view;
	int full_name;
	int size;
	int svg;
	int feature_coverage;
};


FILE* fout;

struct parameters* interface(struct parameters* param,int argc,char **argv);
void start_viewer(struct alignment* aln,struct parameters* param);

struct alignment* hydrophobicity_annotation(struct alignment* aln,int window);

struct alignment* default_dna_annotation(struct alignment* aln);
struct alignment* strong_dna_annotation(struct alignment* aln);
struct alignment* simple_conservation_annotation(struct alignment* aln);


struct alignment* annotation_select(struct alignment* aln,char* ann);

struct alignment* column_annotation(struct alignment* aln,int num_type);
struct alignment* feature_annotation(struct alignment* aln,char* type);
struct alignment* cmsa_annotation(struct alignment* aln);
struct alignment* toby_annotation(struct alignment* aln);
struct alignment* erik_annotation(struct alignment* aln);
struct alignment* blank_annotation(struct alignment* aln);
struct alignment* color_by_average_blosum62_score(struct alignment* aln);
void ki_color_transform(struct alignment* aln);

int get_full_names(struct alignment* aln,int full_name);

void html_out(struct alignment* aln,int ann,int full_name_len,int remove_allgap_columns);
void feature_out(struct alignment* aln,int ann,char* type,int full_name_len,int pretty,int remove_allgap_columns);
void svg_out(struct alignment* aln,int full_name_len,int remove_allgap_columns);
void print_stylesheet();

double* get_stat(struct alignment* aln,double* out);

void viewertop(int size,int view,int full_name_len,char* crosslink);

void viewerbottom(struct alignment* aln,struct parameters* param,int dna);
struct feature* find_free_node(struct feature *n);
void print_ft(struct feature* n);

struct alignment* create_aln(struct alignment* aln);
struct sequence_information* create_si(struct sequence_information* si);
void* tmalloc(int size);
void free_alignment(struct alignment* aln);
void free_parameters(struct parameters* param);
void free_ft(struct feature* n);
void free_utf(struct utype_ufeat* utf);

char* get_input_into_string(char* string,char* infile);

struct alignment* detect_and_read_sequences(struct alignment* si,char* string);

struct alignment* read_macsim_xml_alignment(struct alignment* si,char* string);

struct sequence_information* read_ft(struct sequence_information* si,int sid,char* p);

struct alignment* read_stockholm_alignment(struct alignment* si,char* string);
struct alignment* read_clustal_alignment(struct alignment* si,char* string);
struct alignment* read_fasta_alignment(struct alignment* si,char* string);

struct alignment* remove_gappy_columns(struct alignment* aln);

int byg_detect(char* text,int n);
int byg_count(char* pattern,char*text);
int byg_end(char* pattern,char*text);
int byg_start(char* pattern,char*text);
int foward_to_start(char* p);
int check_identity(char* n,char*m);

void write_fasta_alignment(struct alignment* aln,char* outfile);
void write_unaligned_fasta_alignment(struct alignment* aln,char* outfile,int full_name_len);
void write_macsim_alignment(struct alignment* aln,char* outfile,char* infile);
void write_msf_alignment(struct alignment* aln,char* outfile,int full_name_len);
void write_clustal_alignment(struct alignment* aln,char* outfile);
void write_exact_clustal_alignment(struct alignment* aln,char* outfile);

struct alignment* set_relative_positions_on_features(struct alignment* aln);

struct utype_ufeat* get_unique_features(struct alignment* aln,struct utype_ufeat* utf);
struct feature* add_unique_feature(struct feature *n, struct feature *toadd);
struct feature* add_unique_type(struct feature *n, struct feature *toadd);
struct utype_ufeat* traverse_ft(struct utype_ufeat* utf,struct feature* n);

struct alignment* assign_color_codes(struct alignment* aln);
struct alignment* sort_features(struct alignment* aln);

char* print_ladder(char* ladder,int len);

void feature_coverage(struct alignment* aln,char* type);


struct alignment* phylo (struct alignment* aln,char* input , char* outfile,char* tree_param);


