/*
	msa_viewer_interface.c 
	
	Released under GPL - see the 'COPYING' file   
	
	Copyright (C) 2006 Timo Lassmann <timolassmann@gmail.com>
	
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    
	Please send bug reports, comments etc. to:
	timolassmann@gmail.com
*/

#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include "msa_viewer.h"

struct parameters* interface(struct parameters* param,int argc,char **argv)
{
	int c;
	char *infile = 0;
	int help = 0;
	int quiet = 0;
	
	static char  usage[] ="\n---------------------------------------------------------------------\n\n\
Kalignvu version 2.1, Copyright (C) 2008 Timo Lassmann <timolassmann@gmail.com>\n\
http://msa.sbc.su.se/\n\n\
Usage: kalignvu [options] alignment \n\n\
Options:\n\
	-i	path to alignment.\n\
	-o	print output to file.\n\
	-r	reformats the input alignment. Options include:\n\
		<fasta,msf,clustalw,macsim,raw(unaligned fasta)>\n\
	-n	maximum length of sequence name to be displayed (default 15).\n\
	-s	Starting font-size of the alignment.\n\
	-w	adds a hyperlink to the sequence names.\n\
	-c	prints coverage of each present feature.\n\
	-g	remove columns containing 100% gaps.\n\
	-t	tree.\n\
	-l	tree layout\n\
	-x	svg output.\n\
Examples:\n\
	\n\
	./kalignvu alignment.clu -r fasta > alignment.fasta\n\
	\n\
	- reformats an alignment from clustal to fasta format.\n\
	\n\
	./kalignvu alignment.xml -s 10 -w http://msa.sbc.se/cgi-bin/fetch.cgi > /var/www/html/test.html\n\
	\n\
	- generates the html page 'test.html' \n\
	  containing the alignment in size 10\n\
	  \n\
	  with a hyperlink:\n\
	  \n\
	  http://msa.sbc.su.se/cgi-bin/fetch.cgi?name=<sequence name>\n\
	  from each sequence name.\n\
	  \n\
For further help please contact:\n\
	Timo Lassmann <timolassmann@gmail.com>\n\
	  \n\
---------------------------------------------------------------------\n\n";
	
	

	param = tmalloc(sizeof(struct parameters));
	param->outfile = 0;
	param->input = 0;
	param->reformat = 0;
	param->remove_allgap_columns = 0;
	param->full_name = -1;
	param->size = 12;
	param->ann = 0;
	param->view = 0;
	param->tree = 0;
	param->tree_layout = 0;
	param->svg = 0;
	param->feature_coverage = 0;
	param->crosslink = "";
	param->kalign_parameters = 0;
	while ((c = getopt(argc, argv, "cw:n:i:o:r:hk:s:v:gqxa:tl:")) != -1) {
		switch(c) {
			case 'a':
				param->ann = optarg;
				break;
			case 'c':
				param->feature_coverage = 1;
				break;
			case 'w':
				param->crosslink = optarg;
				break;
			case 'n':
				param->full_name = atoi(optarg);
				break;
			case 'v':
				param->view = atoi(optarg);
				break;
			case 's':
				param->size = atoi(optarg);
				break;
			case 'k':
				param->kalign_parameters = optarg;
				break;
			case 'i':
				infile = optarg;
				break;
			case 'o':
				param->outfile = optarg;
				break;
			case 'r':
				param->reformat = optarg;
				break;
			case 'g':
				param->remove_allgap_columns = 1;
				break;
			case 'h':
				help = 1;
				break;
			case 'x':
				param->svg = 1;
				break;
			case 't':
				param->tree =  1;
				break;
			case 'l':
				param->tree_layout =  optarg;
				break;
			case 'q':
				quiet = 1;
				break;
			default:
				break;
		}
	}
	
	if(quiet){
		fclose(stderr);
	}
	
	
	
	if (optind < argc){
		c = 0;
		while (optind < argc){
			switch(c){
				case 0:
					infile =  argv[optind++];
					break;
				case 1:
					param->outfile =  argv[optind++];
					break;
				default:
					fprintf(stderr,"Unrecognised junk:%s\n",argv[optind++]);
					break;
			}
			c++;
			//fprintf(stderr,"%s\n",argv[optind++]);
		}
	}
	
	if(help){
		fprintf(stdout,"%s\n",usage);
		exit(0);
	}

	param->input = get_input_into_string(param->input,infile);
	if(!param->input){
		fprintf(stdout,"%s\n",usage);
		//fprintf(stderr,"No input detected!\n");
		free_parameters(param);
		exit(0);
	}
	
		
	return param;
}
