/*
	msa_viewer_io.c 
	
	Released under GPL - see the 'COPYING' file   
	
	Copyright (C) 2006 Timo Lassmann <timolassmann@gmail.com>
	
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    
	Please send bug reports, comments etc. to:
	timolassmann@gmail.com
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>

#include "msa_viewer.h"

#define SEEK_START 0
#define SEEK_END 2

struct alignment* detect_and_read_sequences(struct alignment* aln,char* string)
{

	if (byg_start("<macsim>",string) != -1){
		aln = read_macsim_xml_alignment(aln,string);
	}else if (byg_start("CLUSTAL",string) != -1){
		aln = read_clustal_alignment(aln,string);
	}else if (byg_start("PileUp",string) != -1){
		aln = read_clustal_alignment(aln,string);
	}else if (byg_start("MSF:",string) != -1){
		aln = read_clustal_alignment(aln,string);
	}else if (byg_start("STOCKHOLM",string) != -1){
		aln = read_stockholm_alignment(aln,string);
	}else if(byg_count(">",string) > 1){
		aln = read_fasta_alignment(aln,string);
	}
	
	
	
	return aln;
}

char* get_input_into_string(char* string,char* infile)
{
	int i = 0;   
	int string_length = 2;
	char c = 0;    
	FILE *file = 0;
	if(infile){
		if (!(file = fopen( infile, "r" ))){
 			fprintf(stderr,"Cannot open file '%s'\n", infile);
			exit(1);
		}
		if (fseek(file,0,SEEK_END) != 0){
			(void)fprintf(stderr, "ERROR: fseek failed\n");
			(void)exit(EXIT_FAILURE);
		}
		i= ftell (file);
		if (fseek(file,0,SEEK_START) != 0){
			(void)fprintf(stderr, "ERROR: fseek failed\n");
			(void)exit(EXIT_FAILURE);
		}
		string = tmalloc ((i+1)* sizeof(char));
		fread(string,sizeof(char), i, file);
		string[i] = 0;
		fclose(file);
	}else{
		if (!isatty(0)){
			string = malloc(sizeof(char*)*string_length);
			while (!feof (stdin)){
				c = getc(stdin);
				if (i == string_length){
					string_length <<=1;
					string = realloc(string,sizeof(char)*string_length);
				}
				string[i] = c;
				i++;
			}
			string[i-1] = 0;
		}else{
			return 0;
		}
	}
	return string;
}

struct alignment* read_macsim_xml_alignment(struct alignment* aln,char* string)
{
	int c = 0;
	int n = 0;
	int i = 0;
	int j = 0;
	int m = 0;
	int max = 0;
	char *p = 0;

	struct sequence_information* si = 0;

	aln = create_aln(aln);
	aln->numseq = byg_count("<seq-name>",string);
	if(!aln->numseq){
		fprintf(stderr,"No sequences found!\n");
		return 0;
	}

	aln->si  =  tmalloc(sizeof(struct sequence_information* ) * (aln->numseq));

	for (i =0; i < aln->numseq;i++){
		aln->si[i] = create_si(aln->si[i]);
	}
	
	/*p = string;
	if(byg_count("</r></g></seq-data><seq-data><g><r>",p)){
		while((i = byg_start("</r></g></seq-data><seq-data><g><r>",p))!=-1){
			p +=i;
			j = byg_end("</r></g></seq-data><seq-data><g><r>",p);
			for(i = 0; i< j;i++){
				p[i] = ' ';
			}
		}
	}*/
	p = string;
	
	if(byg_count("<g>",p)){
		while((i = byg_start("<g>",p))!=-1){
			p+=i;
			j = byg_end("<r>",p);
			for(i = 0; i< j;i++){
				p[i] = ' ';
			}
			i = byg_start("</r>",p);
			p+=i;
			
			j = byg_end("</g>",p);
			for(i = 0; i< j;i++){
				p[i] = ' ';
			}
			
		}
	}
	
	p = string;
	c = 0;
	
	while((i = byg_end("<sequence",p))!=-1){	
		p+=i;// p1 is at start of entry;
		
		max = byg_end("</sequence>",p);
		
		si = aln->si[c];
		
		i = byg_end("<seq-name>",p);
		if(i < max){
			p +=i; //p1 is at the end of the sequence name tag
			j = byg_start("</seq-name>",p);
		
			
		
			si->namel = j;
			si->name = tmalloc(sizeof(char)*(j+1));
			for (i = 0;i < j;i++){
				si->name[i] = p[i];
			}
			si->name[j] = 0;
		}
		
		i = byg_end("<accession>",p);
		if(i < max){
			p +=i;
			j = byg_start("</accession>",p);
			si->accessionl = j;
			si->accession = tmalloc(sizeof(char)*(j+1));
			for (i = 0;i < j;i++){
				si->accession[i] = p[i];
			}
			si->accession[j] = 0;
		}
		
		
		i = byg_end("<nid>",p);
		if(i < max){
			p +=i;
			j = byg_start("</nid>",p);
			si->nidl = j;
			si->nid = tmalloc(sizeof(char)*(j+1));
			for (i = 0;i < j;i++){
				si->nid[i] = p[i];
			}
			si->nid[j] = 0;
		}
		
		i = byg_end("<ec>",p);
		if(i < max){
			p +=i;
			j = byg_start("</ec>",p);
			si->ecl = j;
			si->ec = tmalloc(sizeof(char)*(j+1));
			for (i = 0;i < j;i++){
				si->ec[i] = p[i];
			}
			si->ec[j] = 0;
		}
		i = byg_end("<group>",p);
		if(i < max){
			p +=i;
			si->group = atoi(p);
		}


		i = byg_end("<ftable>",p);
		if(i < max){
			si = read_ft(si,c,p);
		}
		
		i = byg_end("<length>",p);
		if(i < max){
			p +=i;
			si->abs_length = atoi(p);
		}
		
		i = byg_end("<weight>",p);
		if(i < max){
			p +=i;
			si->weight = atoi(p);
		}
		
		
		i = byg_end("<seq-data>",p);
		if(i < max){
			p+= i;
			j = byg_start("</seq-data>",p);
			si->s = tmalloc(sizeof(char)*(j+1));
			n = 0;
			for (i = 0;i < j;i++){
				if(!iscntrl((int)p[i])&& !isspace((int)p[i])){
					si->s[n] = toupper(p[i]);
					n++;
				}
			}
			si->s[n] = 0;
			si->sl = n;
		}
		c++;
	}
		
	p = string;
	if((i = byg_end("<column-score>",p)) != -1){ 
		c = 0;
		//p += i; 
		j = byg_count("<column-score>",p);
		aln->numcolumn = j;
		aln->ca = tmalloc(sizeof(struct column_ann*)*j);
		
		while((i = byg_end("<column-score>",p))!=-1){
			p+=i;
			aln->ca[c] = 0;
			aln->ca[c] = tmalloc(sizeof(struct column_ann));
			i = byg_end("<colsco-name>",p);
			p+=i;
			j = byg_start("</colsco-name>",p);
			aln->ca[c]->name = tmalloc(sizeof(char)*(j+1));
			for (i = 0;i < j;i++){
				aln->ca[c]->name[i] = p[i];
			}
			aln->ca[c]->name[j] =0; 
			
			i = byg_end("<colsco-owner>",p);
			p+=i;
			aln->ca[c]->owner = atoi(p);			
			
			aln->ca[c]->doublec = tmalloc(sizeof(double)*(aln->si[0]->sl+1));
			
			i = byg_end("<colsco-data>",p);
			p+=i;
			m = byg_end("</colsco-data>",p);
			
			
			n = 0;
			j = 0;
			
			while(n < m){
				aln->ca[c]->doublec[j] = atof(p);
				if((i = byg_end(" ",p))!= -1){
					n+=i;
					p+=i;
					j++;
				}else{
					break;
				}
			}
			p-=i;
			c++;
			
		}
		//c++;
		/*
		
		c = 0;
		p += i; 
		j = byg_count("<colsco-name>",p);
		aln->numcolumn = j;
		aln->ca = tmalloc(sizeof(struct column_ann*)*j);
		while((i = byg_end("<colsco-name>",p))!=-1){
			aln->ca[c] = 0;
			aln->ca[c] = tmalloc(sizeof(struct column_ann));
			p+=i;
			
			j = byg_start("</colsco-name>",p);
			aln->ca[c]->name = tmalloc(sizeof(char)*(j+1));
			for (i = 0;i < j;i++){
				aln->ca[c]->name[i] = p[i];
			}
			aln->ca[c]->name[j] =0; 
			
			i = byg_end("<colsco-owner>",p);
			p+=i;
			aln->ca[c]->owner = atoi(p);			
			
			aln->ca[c]->doublec = tmalloc(sizeof(double)*(aln->si[0]->sl+1));
			
			i = byg_end("<colsco-data>",p);
			p+=i;
			m = byg_end("</colsco-data>",p);
			n = 0;
			j = 0;
			
			while(n < m){
				aln->ca[c]->doublec[j] = atof(p);
				if((i = byg_end(" ",p))!= -1){
					n+=i;
					p+=i;
					j++;
				}else{
					break;
				}
			}
			c++;
			
		}*/
		
	}
	/*for (i = 0; i< aln->numcolumn;i++){
		fprintf(stderr,"Column_score:%d	name:%s\n",i,aln->ca[i]->name);
		fprintf(stderr,"Column_score:%d	Owner:%d\n",i,aln->ca[i]->owner);
		fprintf(stderr,"%s\n",aln->ca[i]->name);	
		for(j=0;j < aln->si[0]->sl;j++){
			fprintf(stderr,"%0.0f ",aln->ca[i]->doublec[j]);
		}
		fprintf(stderr,"\n");
	}*/
	free(string);
	return aln;
}

struct sequence_information* read_ft(struct sequence_information* si,int sid,char* p)
{

	int i,j;
	struct feature *n = 0;
	struct feature *old_n = 0;
	char* p1 = 0;
	p1 = p;
	//<fitem><ftype>STRUCT</ftype><fstart>17</fstart><fstop>30</fstop><fcolor>3</fcolor><fscore>0.00</fscore><fnote>HELIX</fnote></fitem>
	while((j = byg_end("<fitem>",p1))!= -1){
		i = byg_end("</seq-info>",p1);
		
		if(j >i){
			break;
		}

		n = tmalloc(sizeof(struct feature));
		n->next = 0;

		p1+=j;// p1 is at start of entry;
		i = byg_end("<ftype>",p1);
		p1 +=i; //p1 is at the end of the sequence name tag
		p1+= foward_to_start(p1);
		j = byg_start("</ftype>",p1);

		n->type = tmalloc(sizeof(char*)*(j+1));
		for (i = 0; i < j;i++){
			n->type[i] = p1[i];
		}
		n->type[j] = 0;
		
		i = byg_end("<fstart>",p1);
		p1+= i;
		p1+= foward_to_start(p1);
		/*j = byg_start("</fstart>",p1);
		
		for (i = 0; i < j;i++){
			tmp[i] = p1[i];
		}
		tmp[j] = 0;*/
		n->start = atoi(p1);
		
		i = byg_end("<fstop>",p1);
		p1+= i;
		p1+= foward_to_start(p1);
		/*j = byg_start("</fstop>",p1);
		for (i = 0; i < j;i++){
			tmp[i] = p1[i];
		}
		tmp[j] = 0;*/
		n->end = atoi(p1);

		i = byg_end("<fcolor>",p1);
		if(i){
			p1+= i;
			p1+= foward_to_start(p1);
			n->color = atoi(p1);
		}

		i = byg_end("<fnote>",p1);
		p1+= i;
		p1+= foward_to_start(p1);
		j = byg_start("</fnote>",p1);
		n->note = tmalloc(sizeof(char*)*(j+1));
		for (i = 0; i < j;i++){
			n->note[i] = p1[i];
		}
		
		n->note[j] = 0;
		
		if(n->start >= 0 && n->end >= 0){
			if((old_n = si->ft)!= 0){
				while(old_n->next!=0){
					old_n = old_n->next;
				}
				old_n->next = n;
			}else{
				si->ft = n;
			}
		}else{
			free(n->type);
			free(n->note);
			free(n);
		}
		
		n = 0;
	}
	return si;
}

struct alignment* read_stockholm_alignment(struct alignment* aln,char* string)
{
	int c = 0;
	int n = 0;
	int i = 0;
	int j = 0;
	char *p = 0;
	struct sequence_information* si = 0;


	aln = create_aln(aln);

	p = string;
	while((i = byg_end("\n",p))!=-1){
		p+=i;
		if (!(byg_start("//",p))){
			break;
		}	
		j = byg_end("#",p);
		if(j != 1){
			aln->numseq++;
		}
	}
	

	aln->si  =  tmalloc(sizeof(struct sequence_information* ) * (aln->numseq));
	for(i = 0; i < aln->numseq;i++){
		aln->si[i] = create_si(aln->si[i]);
	}
		
	p = string;
	while((i = byg_end("\n",p))!=-1){
		if(c < aln->numseq){
			si = aln->si[c];
		}
		
		p+=i;
		if (!(byg_start("//",p))){
			break;
		}	
		j = byg_end("#",p);
		if(j != 1){
			j = byg_start(" ",p);
			si->namel = j;
			si->name = tmalloc(sizeof(char)*(j+1));
			for (i = 0;i < j;i++){
				si->name[i] = p[i];
			}
			si->name[j] = 0;
			
			p+=j;
			j = byg_start("\n",p);

			si->s =  tmalloc(sizeof(char)*(j+1));;
			n = 0;
			for (i = 0;i < j;i++){
				if(!iscntrl((int)p[i]) && !isspace((int)p[i]) ){
					si->s[n] =  toupper(p[i]);
					n++;
				}
			}
			si->s[n] = 0;
			si->sl = n;
			c++;
		}

	}
	free(string);
	return aln;
}

struct alignment* read_clustal_alignment(struct alignment* aln,char* string)
{
	int c = 0;
	int n = 0;
	int len = 0;
	int i = 0;
	int j = 0;
	char *p = 0;

	struct sequence_information* si = 0;

	aln = create_aln(aln);

	p = string;
	
	while((i = byg_end("\n",p))!=-1){
		p+=i;
		j = byg_end(" ",p);
		n = byg_end("\n",p);
		if(n > 2 && n>j && j!= 1){
			if(c ==0){
				i = j;
				while(p[i] != '\n'){
					if(!iscntrl((int)p[i]) && !isspace((int)p[i]) ){
						len++;
					}
					i++;
				}		
			}
			c++;
		}else{
			if (c){
				if(c > aln->numseq){
					aln->numseq = c;
				}
				c =0;
			}
		}
	}
	
	aln->si  =  tmalloc(sizeof(struct sequence_information* ) * (aln->numseq));
		
	for(i =0;i < aln->numseq;i++){
		aln->si[i] = create_si(aln->si[i]);
		aln->si[i]->s = tmalloc(sizeof(char)*(len+1));
	}
	
	p = string;
	c = 0;
	
	while((i = byg_end("\n",p))!=-1){
	
		if(c < aln->numseq){
			si = aln->si[c];
		}
		
		p+=i;
		j = byg_end(" ",p);
		n = byg_end("\n",p);
		if(n > 2 && n>j && j!= 1){
			if(si->namel == 0){
				si->namel = j;
				si->name = tmalloc(sizeof(char)*(j+1));
				for (i = 0;i < j;i++){
					si->name[i] = p[i];
				}
				si->name[j] = 0;
			}
			
			
			for (i = j;i < n;i++){
				if(!iscntrl((int)p[i])&& !isspace((int)p[i])){
					si->s[si->sl] = toupper(p[i]);
					si->sl++;
				}
			}		
			c++;
		}else{
			if (c){
				c =0;
			}	
		}
	}
	
	for (i = 0; i < aln->numseq;i++){
		aln->si[i]->s[aln->si[i]->sl] = 0;
	}
	free(string);
	return aln;
}

struct alignment* read_fasta_alignment(struct alignment* aln,char* string)
{
	int c = 0;
	int n = 0;
	int i = 0;
	int j = 0;
	int stop = 0;
	int nbytes;										// O	12				//U17
		
	nbytes = strlen(string);
	
	aln = create_aln(aln);

	for (i =0;i < nbytes;i++){
		if (string[i] == '>'&& stop == 0){
			stop = 1;
			aln->numseq++;
		}
		if (string[i] == '\n'){
			stop = 0;
		}
	}
	if(!aln->numseq){
		fprintf(stderr,"No sequences found!\n");
		return 0;
	}

	aln->si  =  tmalloc(sizeof(struct sequence_information* ) * (aln->numseq+1));
	for (i = 0;i <= aln->numseq;i++){
		aln->si[i] = create_si(aln->si[i]);
	}
	
	j = 0;
	for (i =0;i < nbytes;i++){
		if (string[i] == '>' && stop == 0){
			stop = 1;
			aln->si[j]->sl = c;
			j++;
			c = 0;
		}
		if (string[i] == '\n'){
			if(stop == 1){
				aln->si[j-1]->namel = n;
				n = 0;
			}
			stop = 0;
		}
		if (stop == 1 && string[i] != '\n' && string[i] != 0 && string[i] != '>' ){
			n++;
		}
		if (stop == 0 && string[i] != '\n' && string[i] != 0 ){
			if(!iscntrl((int)string[i])){
				c++;
			}	

		}
	}

	aln->si[j]->sl = c;
	for (i =1;i < aln->numseq+1;i++){
		if(!aln->si[i]->sl){
			fprintf(stderr,"Sequence %d has a length of 0!!\n",i-1);
			exit(1);
		}

		aln->si[i-1]->sl = aln->si[i]->sl;
	}
	free(aln->si[aln->numseq]);
	for (i = aln->numseq;i--;){
		aln->si[i]->s = tmalloc(sizeof(char)*(aln->si[i]->sl+1));
		aln->si[i]->name = tmalloc(sizeof(char)*(aln->si[i]->namel+1));
	}

	
	stop = 0;
	j =0;
	for (i =0;i < nbytes;i++){
		if (string[i] == '>' && stop == 0 ){
			stop = 1;
			j++;
			c = 0;
		}
		if (string[i] == '\n'){
			if(stop == 1){
				n = 0;
			}
			stop = 0;
		}
		if (stop == 1 && !iscntrl((int)string[i]) && string[i] != '>' ){
			aln->si[j-1]->name[n] = string[i];
			n++;
		}
		if (stop == 0 && string[i] != '\n' && string[i] != 0 ){
			if(!iscntrl((int)string[i])){
				aln->si[j-1]->s[c] = toupper(string[i]);
				c++;
			}
		}
	}

	for (i =0;i< aln->numseq;i++){
		aln->si[i]->s[aln->si[i]->sl] = 0;
		aln->si[i]->name[aln->si[i]->namel] = 0;
	}	
	
	free(string);
	return aln;
}

void write_fasta_alignment(struct alignment* aln,char* outfile)
{
	int i,j,c;

	for (i =0;i< aln->numseq;i++){
		fprintf(fout,">%s\n",aln->si[i]->name);
		c = 1;
		for(j = 0; j < aln->aln_len;j++){
			fprintf(fout,"%c",aln->si[i]->s[j]);
			if (c == 60){
				fprintf(fout,"\n");
				c = 0;
			}
			c++;
		}
		fprintf(fout,"\n");
	}
	free_alignment(aln);
}


void write_unaligned_fasta_alignment(struct alignment* aln,char* outfile,int full_name_len)
{
	int i,j,c;
	
	int* print_name_len = 0;
	
	if(full_name_len == -1){
		full_name_len = 10000000;
	}
	
	print_name_len = malloc(sizeof(int)*aln->numseq);
	for (i = 0; i< aln->numseq;i++){
		c = byg_start("untitled",aln->si[i]->name);
		if(c != -1){
			aln->si[i]->namel  = c;
		}
	
		print_name_len[i] = (aln->si[i]->namel < full_name_len) ? aln->si[i]->namel  :full_name_len;
	}

	
	for (i =0;i< aln->numseq;i++){
		fprintf(fout,">");
		for (j = 0; j < print_name_len[i];j++){
			fprintf(fout,"%c",aln->si[i]->name[j]);
		}
		fprintf(fout,"\n");
		
		
		c = 1;
		for(j = 0; j < aln->aln_len;j++){
			if(isalpha((int)aln->si[i]->s[j])){
				fprintf(fout,"%c",aln->si[i]->s[j]);
			}
			if (c == 60){
				fprintf(fout,"\n");
				c = 0;
			}
			if(isalpha((int)aln->si[i]->s[j])){
				c++;
			}
		}
		fprintf(fout,"\n");
	}

	free(print_name_len);
	free_alignment(aln);
}

void write_clustal_alignment(struct alignment* aln,char* outfile)
{
	int i,j;
	int c = 0;
	int max = 0;
	
	fprintf(fout,"Alignment in CLUSTAL W (1.83) format\n\n\n");
	
	for (j = 0; j< aln->numseq;j++){		
		if( aln->si[j]->namel > max){
			max = aln->si[j]->namel;
		}
	}
		
	for (i = 0; i+60 < aln->aln_len;i +=60){
		for (j = 0; j< aln->numseq;j++){
			fprintf(fout,"%s",aln->si[j]->name);
			c = aln->si[j]->namel;
			while(c < max+3){
				fprintf(stdout," ");
				c++;
			}
			
			for (c = 0; c< 60;c++){				
				fprintf(fout,"%c",aln->si[j]->s[c+i]);
			}
			fprintf(fout,"\n");
			
		}
		fprintf(fout,"\n\n");
	}
	for (j = 0; j < aln->numseq;j++){
		fprintf(fout,"%s",aln->si[j]->name);
		c = aln->si[j]->namel;
		while(c < max+3){
			fprintf(stdout," ");
			c++;
		}
	
		for (c = i; c< aln->aln_len;c++){
			fprintf(fout,"%c",aln->si[j]->s[c]);
		}
		fprintf(fout,"\n");
	
	}
	fprintf(fout,"\n\n");
	
	free_alignment(aln);
}

void write_exact_clustal_alignment(struct alignment* aln,char* outfile)
{
	int i,j;
	int c = 0;
	int max = 0;
	

	fprintf(fout,"CLUSTAL W(1.81) multiple sequence alignment\n\n\n");
	
	for (j = 0; j< aln->numseq;j++){		
		if( aln->si[j]->namel > max){
			max = aln->si[j]->namel;
		}
	}
		
	for (i = 0; i+60 < aln->aln_len;i +=60){
		for (j = 0; j< aln->numseq;j++){
			fprintf(fout,"%s",aln->si[j]->name);
			c = aln->si[j]->namel;
			while(c < max+3){
				fprintf(fout," ");
				c++;
			}
			
			for (c = 0; c< 60;c++){				
				fprintf(fout,"%c",aln->si[j]->s[c+i]);
			}
			fprintf(fout,"\n");
			
		}
		fprintf(fout,"\n\n");
	}
	for (j = 0; j < aln->numseq;j++){
		fprintf(fout,"%s",aln->si[j]->name);
		c = aln->si[j]->namel;
		while(c < max+3){
			fprintf(fout," ");
			c++;
		}
	
		for (c = i; c< aln->aln_len;c++){
			fprintf(fout,"%c",aln->si[j]->s[c]);
		}
		fprintf(fout,"\n");
	
	}
	fprintf(fout,"\n\n");
	
	free_alignment(aln);
}



void write_msf_alignment(struct alignment* aln,char* outfile,int full_name_len)
{
	int i,j,f;
	int max = 0;
	int c = 0;

	int* print_name_len = 0;

	for (i =0; i < aln->numseq;i++){
		fprintf(stderr,"%d\n",aln->si[i]->sl);
	}
	if(full_name_len == -1){
		full_name_len = 10000000;
	}
	
	print_name_len = malloc(sizeof(int)*aln->numseq);
	for (i = 0; i< aln->numseq;i++){
		c = byg_start("untitled",aln->si[i]->name);
		if(c != -1){
			aln->si[i]->namel  = c;
		}
	
		print_name_len[i] = (aln->si[i]->namel < full_name_len) ? aln->si[i]->namel  :full_name_len;
	}
	
	fprintf(fout,"PileUp\n\n\n\n   MSF:   %d  Type: P    Check:  ???   ..\n\n",aln->si[0]->sl);

	for (j = 0; j< aln->numseq;j++){
		if( aln->si[j]->namel > max){
			max = aln->si[j]->namel;
		}
	}
	if(max > full_name_len){
		max = full_name_len;
	}

	
	for (i = 0; i< aln->numseq;i++){
			fprintf(fout," Name: ");
			for (j = 0; j < print_name_len[i];j++){
				fprintf(fout,"%c",aln->si[i]->name[j]);
			}
			
			c = print_name_len[i];
			while(c < max+3){
				fprintf(fout," ");
				c++;
			}
			fprintf(fout,"Len:   ");
			fprintf(fout,"%d",aln->si[i]->sl);
			
			fprintf(fout,"  Check:  ???  Weight:  ???\n");
			
	}
	fprintf(fout,"\n\n//\n\n");
	
	for (i = 0; i+60 < aln->aln_len;i +=60){
		for (j = 0; j< aln->numseq;j++){
			//fprintf(stdout,"%s",aln->si[j]->name);
			
			for (c = 0; c < print_name_len[j];c++){
				fprintf(fout,"%c",aln->si[j]->name[c]);
			}
			//fprintf(stdout,"\n");
			
			c = print_name_len[j] ;
			while(c < max+3){
				fprintf(fout," ");
				c++;
			}
			f = 0;
			for (c = 0; c< 60;c++){
				
				
				if (f == 10){
					fprintf(fout," ");
					f = 0;
				}
				if (isalpha((int)aln->si[j]->s[c+i])){
					fprintf(fout,"%c",aln->si[j]->s[c+i]);
				}else{
					fprintf(fout,"-");
				}
				f++;
			}
			fprintf(fout,"\n");
			
		}
		fprintf(fout,"\n\n");
	}
	for (j = 0; j < aln->numseq;j++){
		//fprintf(stdout,"%s",aln->si[j]->name);
		for (c = 0; c < print_name_len[j];c++){
			fprintf(fout,"%c",aln->si[j]->name[c]);
		}
		//c = aln->si[j]->namel;
		c = print_name_len[j];
		while(c < max+3){
			fprintf(fout," ");
			c++;
		}
		f = 0;
		for (c = i; c< aln->aln_len;c++){
			
			if (f == 10){
				fprintf(fout," ");
				f = 0;
			}
			//fprintf(stdout,"%c",aln->si[j]->s[c]);
			if (isalpha((int)aln->si[j]->s[c])){
				fprintf(fout,"%c",aln->si[j]->s[c]);
			}else{
				fprintf(fout,"-");
			}
			f++;
		}
		fprintf(fout,"\n");
	
	}
	fprintf(fout,"\n\n");

	free(print_name_len);
	free_alignment(aln);
}



void write_macsim_alignment(struct alignment* aln,char* outfile,char* infile)
{
	int i,j;
	struct feature *n = 0;
	
	fprintf(fout,"<?xml version=\"1.0\"?>\n<!DOCTYPE macsim SYSTEM \"http://www-bio3d-igbmc.u-strasbg.fr/macsim.dtd\">\n<macsim>\n<alignment>\n<aln-name>");
	if(infile){
		fprintf(fout,"%s.macsim",infile);
	}else{
		fprintf(fout,"Alignment re-formatted by Kalignvu");
	}
	fprintf(fout,"</aln-name>\n");

	for (i =0;i< aln->numseq;i++){
		fprintf(fout,"<sequence seq-type=\"Protein\">\n");
		fprintf(fout,"<seq-name>%s</seq-name>\n",aln->si[i]->name);
		if(aln->si[i]->ft){
			fprintf(fout,"<seq-info>\n");
			fprintf(fout,"<accession>%s</accession>\n",aln->si[i]->accession);
			fprintf(fout,"<nid>%s</nid>\n",aln->si[i]->nid);
			fprintf(fout,"<ec>%s</ec>\n",aln->si[i]->ec);
			fprintf(fout,"<group>%d</group>\n",aln->si[i]->group);
			fprintf(fout,"<ftable>\n");
			n = aln->si[i]->ft;
			while(n){
				fprintf(fout,"<fitem><ftype>%s</ftype><fstart>%d</fstart><fstop>%d</fstop><fnote>%s</fnote></fitem>\n",n->type,n->start,n->end,n->note);
				n = n->next;
			}
			fprintf(fout,"</ftable>\n</seq-info>\n");
		}
		fprintf(fout,"<seq-data>\n");
		for ( j = 0 ; j < aln->aln_len;j++){
			fprintf(fout,"%c",aln->si[i]->s[j]);
		}
		fprintf(fout,"\n");
		fprintf(fout,"</seq-data>\n");
		fprintf(fout,"</sequence>\n");
	}
	for(i =0;i< aln->numcolumn;i++){
		fprintf(fout,"<column-score>");
		fprintf(fout,"<colsco-name>%s</colsco-name>",aln->ca[i]->name);
		fprintf(fout,"<colsco-owner>%d</colsco-owner>",aln->ca[i]->owner);
		fprintf(fout,"<colsco-type>int</colsco-type>");
		fprintf(fout,"<colsco-data>");
		for (j = 0;j < aln->si[0]->sl;j++){
			fprintf(fout,"%f	",aln->ca[i]->doublec[j]);
		}
		fprintf(fout,"</colsco-data></column-score>");
	
	}
	
	
	fprintf(fout,"</alignment>\n");
	fprintf(fout,"</macsim>\n");
	free_alignment(aln);
}





