/*
	msa_viewer_main.c 
	
	Released under GPL - see the 'COPYING' file   
	
	Copyright (C) 2006 Timo Lassmann <timolassmann@gmail.com>
	
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    
	Please send bug reports, comments etc. to:
	timolassmann@gmail.com
*/


#include <getopt.h>
#include <string.h>
#include <ctype.h>
#include "msa_viewer.h"

int main(int argc,char** argv)
{
	int i,j;
	struct parameters* param = 0; 
	param = interface(param,argc,argv);
	
	struct alignment *aln = 0;
	aln = detect_and_read_sequences(aln,param->input);



	if(aln){
		for (i =0; i < aln->numseq-1;i++){
			for (j = i+ 1; j < aln->numseq;j++){
				if(aln->si[i]->sl != aln->si[j]->sl){
					//fprintf(stderr,"%d	%d	%s\n",aln->si[i]->sl,aln->si[j]->sl,aln->si[j]->name);
					fprintf(stderr,"Sequences seem to be unaligned. Use Kalign to align them first!\n");
					free_alignment(aln);
					free_parameters(param);
					exit(1);
				}
			}
		}
		aln->aln_len = aln->si[0]->sl;
	}else{
		if (byg_count("(",param->input) == byg_count(")",param->input)){
			//fprintf(stderr,"Tree assumed.\n");
			if(!param->tree){
				param->tree = 1;			
			}
			if(!param->tree_layout){
				param->tree_layout = "upgmacir";
			}
		}
	}
	
	if(param->outfile){
		if ((fout = fopen(param->outfile, "w")) == NULL){
			fprintf(stderr,"can't open output\n");
			exit(0);
		}
	}else{
		fout = stdout;
	}

	if(!aln){
		 aln = phylo(aln,param->input,param->outfile,param->tree_layout);
	}else if(aln && param->tree){
		//if(aln){
			aln = phylo(aln,0,param->outfile,param->tree_layout);
			free_alignment(aln);
		//}else{
		//	 aln = phylo(aln,param->input,param->outfile,param->tree);
		//}
	}else if(param->reformat){
		aln->numtypes = 0;
		if (byg_start("macsim",param->reformat) != -1){
			write_macsim_alignment(aln,param->outfile,0);
		}else if(byg_start("msf",param->reformat) != -1){
			write_msf_alignment(aln,param->outfile,param->full_name);
		}else if(byg_start("eclu",param->reformat) != -1){
			write_exact_clustal_alignment(aln,param->outfile);
		}else if(byg_start(param->reformat,"clustalCLUSTALclustalwCLUSTALW") != -1){
			write_clustal_alignment(aln,param->outfile);
		}else if(byg_start(param->reformat,"unalignedUNALIGNEDrawRAW") != -1){
			write_unaligned_fasta_alignment(aln,param->outfile,param->full_name);
		}else{
			write_fasta_alignment(aln,param->outfile);
		}
	}else if(param->svg){
		//aln = color_by_average_blosum62_score(aln);
		//ki_color_transform(aln); 

		if(param->ann){
			aln =  annotation_select(aln,param->ann);
		}else{
			aln = color_by_average_blosum62_score(aln);
		}
		param->full_name = get_full_names(aln,param->full_name);
		
		svg_out(aln,param->full_name,param->remove_allgap_columns);
		
	}else{
		start_viewer(aln,param);
	}
	if(param->outfile){
		fclose(fout);
	}
	
	free_parameters(param);
	return 1;
}

void start_viewer(struct alignment* aln,struct parameters* param)
{
	char* input = param->input;
	//char* outfile = param->outfile;
	int i,c;
	int dna = 0;
	//int max = -1; 
	if (!input){
		fprintf(stdout,"Could not read input string\n");
		exit(1);
	}
	
	for (i = 0;i < aln->numseq;i++){
		if(aln->si[i]->ft){
			aln->numtypes = 0;
			aln = sort_features(aln);
			aln = assign_color_codes(aln);
			aln = set_relative_positions_on_features(aln);
			break;
		}
	}

	if(param->feature_coverage){
		for(i = 0; i< aln->numtypes;i++){
			feature_coverage(aln,aln->types[i]);
		}
		free_alignment(aln);
		free_parameters(param);
		exit(1);
	}
	
	
	/*if(outfile){
		if ((fout = fopen(outfile, "w")) == NULL){
			fprintf(stderr,"can't open output\n");
			exit(0);
		}
	}
	*/

	for (i = 0; i < aln->numseq;i++){
		dna = byg_detect(aln->si[i]->s,aln->si[i]->sl);
		if(dna){
		//	fprintf(stderr,"\n \n DETECTED DNA:%s\n",aln->si[i]->s);
			break;
		}
	}

	/*for (i = 0; i < aln->numseq;i++){
		if( aln->si[i]->namel > max){
			max = aln->si[i]->namel;
		}
	}
	
	if(param->full_name == -1){
		param->full_name = 15;
	}
	
	if(max < param->full_name){
		param->full_name = max+1;
	//	for (j = 0; j< aln->numseq;j++){
	//		 aln->si[j]->name[max] = 0;
	//	}
	}
	if(param->full_name < 7) param->full_name = 7;*/
	param->full_name = get_full_names(aln,param->full_name);
	//fprintf(stderr,"name to be shown %d\n",param->full_name);
	
	viewertop(param->size,param->view,param->full_name+1,param->crosslink);

	c = 0;
	
	if(!dna){
		if(param->ann){
			aln =  annotation_select(aln,param->ann);
			html_out(aln,c,param->full_name,param->remove_allgap_columns);
			c++;
		}else if(aln->numseq > 100){
			aln = blank_annotation(aln);
			html_out(aln,c,param->full_name,param->remove_allgap_columns);
			c++;
		}else if(aln->numseq > 50){
			aln = color_by_average_blosum62_score(aln);
			html_out(aln,c,param->full_name,param->remove_allgap_columns);
			c++;
		}else{
			aln = color_by_average_blosum62_score(aln);
			html_out(aln,c,param->full_name,param->remove_allgap_columns);
			c++;

			aln = toby_annotation(aln);
			html_out(aln,c,param->full_name,param->remove_allgap_columns);
			c++;

			aln = erik_annotation(aln);
			html_out(aln,c,param->full_name,param->remove_allgap_columns);
			c++;
		}

		//hydrophobicity
		aln =  hydrophobicity_annotation(aln,7);
		html_out(aln,c,param->full_name,param->remove_allgap_columns);
		c++;
		aln =  hydrophobicity_annotation(aln,21);
		html_out(aln,c,param->full_name,param->remove_allgap_columns);
		c++;
		
		for(i = 0; i< aln->numtypes;i++){
			
			if(check_identity("CMSA",aln->types[i])!= -1){
				aln = cmsa_annotation(aln);
				feature_out(aln,c,aln->types[i],param->full_name,1,param->remove_allgap_columns);
			}else{
				aln = feature_annotation(aln,aln->types[i]);
				feature_out(aln,c,aln->types[i],param->full_name,0,param->remove_allgap_columns);
			}
			//feature_out(aln,c,aln->types[i],param->full_name,0);
			c++;
		}
	
		for(i = 0; i< aln->numcolumn;i++){
			aln = column_annotation(aln,i);
			html_out(aln,c,param->full_name,param->remove_allgap_columns);
			c++;
		}
	}else{
		if(!param->ann){
		if(aln->numseq > 1000){
			aln = blank_annotation(aln);
			html_out(aln,c,param->full_name,param->remove_allgap_columns);
			c++;
		}else{
			aln = default_dna_annotation(aln);
			html_out(aln,c,param->full_name,param->remove_allgap_columns);
			c++;
			aln = strong_dna_annotation(aln);
			html_out(aln,c,param->full_name,param->remove_allgap_columns);
			c++;
				
			aln = simple_conservation_annotation(aln);
			html_out(aln,c,param->full_name,param->remove_allgap_columns);
			c++;
			
			
		}
		
		
		for(i = 0; i< aln->numtypes;i++){
			//fprintf(stderr,"%s\n",aln->types[i]);
			if(check_identity("CMSA",aln->types[i])!= -1){
				aln = cmsa_annotation(aln);
				feature_out(aln,c,aln->types[i],param->full_name,1,param->remove_allgap_columns);
			}else{
		
				aln = feature_annotation(aln,aln->types[i]);
				feature_out(aln,c,aln->types[i],param->full_name,0,param->remove_allgap_columns);
			}
			c++;
		}
		}else{
			aln =  annotation_select(aln,param->ann);
			feature_out(aln,c,aln->types[i],param->full_name,0,param->remove_allgap_columns);
			c++;
		}
	}
	
	//print_stylesheet();

	viewerbottom(aln,param,dna);

	/*if(outfile){
		fclose(fout);
	}*/

	free_alignment(aln);
}
