/*
	msa_viewer_mem.c 
	
	Released under GPL - see the 'COPYING' file   
	
	Copyright (C) 2006 Timo Lassmann <timolassmann@gmail.com>
	
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    
	Please send bug reports, comments etc. to:
	timolassmann@gmail.com
*/

#include <stdlib.h>
#include <stdio.h>
#include "msa_viewer.h"


struct alignment* create_aln(struct alignment* aln)
{
	int i;
	aln = (struct alignment *) tmalloc(sizeof(struct alignment));
	aln->ca = 0; 
	aln->si = 0;
	aln->ann = 0;
	aln->numseq = 0;
	aln->aln_len = 0;
	aln->types = 0;
	aln->numtypes = 0;
	aln->numcolumn = 0;
	aln->ann_key = malloc(sizeof(int)*10);
	for (i = 0;i < 10;i++){
		aln->ann_key[i] = 0;
	}
	
	
	return aln;
}

struct sequence_information* create_si(struct sequence_information* si)
{
	si = tmalloc(sizeof(struct sequence_information));
	si->ft = 0; 
	si->s = 0;
	si->name = 0;
	si->accession = 0;
	si->nid = 0;
	si->ec = 0;
	si->sl = 0;
	si->namel = 0;
	si->accessionl = 0;
	si->nidl = 0;
	si->ecl = 0;
	si->group = 0;
	si->abs_length = 0;
	si->weight = 0;
	return si;
}



void* tmalloc(int size)
{
	void* p;
	p = (void*)malloc(size);
	if (!p){
		fprintf(stderr,"Out of memory!\n");
		exit(0);
	}
	return p;
}

void free_utf(struct utype_ufeat* utf)
{
	free_ft(utf->t);
	free_ft(utf->f);
	free(utf);
}

void free_parameters(struct parameters* param)
{
	free(param);
}

void free_alignment(struct alignment* aln)
{
	int i;
	if(aln->numcolumn){
		for (i = aln->numcolumn;i--;){
			free(aln->ca[i]->doublec);
			free(aln->ca[i]->name);
			free(aln->ca[i]);
			
		}
		free(aln->ca);
	}
	if(aln->ann){
		for (i = aln->numseq;i--;){
			free(aln->ann[i]);
		}
		free(aln->ann);
	}

	for (i =0; i < aln->numseq;i++){
		if (aln->si[i]->ft){
			free_ft(aln->si[i]->ft);
		}
		free(aln->si[i]->s);
		free(aln->si[i]->name);
		free(aln->si[i]->ec);
		free(aln->si[i]->nid);
		free(aln->si[i]->accession);
		
		free(aln->si[i]);
	}
	free(aln->si);
	free(aln->ann_key);
	if(aln->numtypes){
		for (i = aln->numtypes;i--;){
			free(aln->types[i]);
		}
		free(aln->types);
	}
	free(aln);
}


void free_ft(struct feature* n)
{
	struct feature* old_n = 0;
	 if (n != NULL){
	 	old_n = n;
	 	n= n ->next;
	 	free(old_n->type);
	 	free(old_n->note);
 	 	free(old_n);
		free_ft(n);
	}
}
