/*
	msa_viewer_misc.c 
	
	Released under GPL - see the 'COPYING' file   
	
	Copyright (C) 2006 Timo Lassmann <timolassmann@gmail.com>
	
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    
	Please send bug reports, comments etc. to:
	timolassmann@gmail.com
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "msa_viewer.h"

struct alignment* set_relative_positions_on_features(struct alignment* aln)
{
	struct feature* n = 0;
	int* pos = 0;
	int i,j,c;
	pos = tmalloc(sizeof(int)*aln->aln_len);
	for (i = 0;i < aln->numseq;i++){
		c =1;
		for(j =0; j < aln->aln_len;j++){
	//		if(isalpha((int)aln->s[i][j])){
			if(isalpha((int)aln->si[i]->s[j])){
				pos[j] = c;
				c++;
			}else{
				pos[j] = -1;
			}
		}
		//for(j =0; j < aln->aln_len;j++){
		//	fprintf(stderr,"%d	%d\n",j,pos[j]);
		//}
		//fprintf(stderr,"\n");
		
		j = 0;
	//	n = aln->ft[i];
		n = aln->si[i]->ft;
		while(n){
			//fprintf(stderr,"SET:%s	%s	%d	%d\n",n->type,n->note,n->start,n->end);
			//printf("SEQ:%d	OLD:%d	%d	%d\n",i,n->start,n->end,j);
			j = 0;
			
			if(n->start > 0){
				while(pos[j] != n->start){
				//	fprintf(stderr,"%d	%d	%d\n",j,pos[j],n->start);
				
					j++;
					if(j >= aln->si[i]->sl){
						j = -1;
						break;
					}
				}
			}else{
				j = -1;
			}
			if(j!= -1 && n->end > 0 ){
			//	n->start = j;
				n->rel_start = j;
				j = 0;
				while(pos[j] != n->end){
					j++;
					if(j >= aln->si[i]->sl){
						j = aln->si[i]->sl -1;
						break;
					}
				}
			//	n->end = j;
				n->rel_end = j;
			//	printf("SEQ:%d	nEW:%d	%d	%d\n",i,n->start,n->end,j);
			}else{
				n->rel_start = -1;
				n->rel_end = -10;
			}
			//fprintf(stderr,"SET:%s	%s	%d	%d\n\n",n->type,n->note,n->rel_start,n->rel_end);
			n = n->next;
		}
	}
	free(pos);
	return aln;
}

struct alignment* sort_features(struct alignment* aln)
{
	struct feature* n = 0;
	struct feature* old_n = 0;
	int i;
	int tmp;
	char* t;

	for (i = 0;i < aln->numseq;i++){
	//	n = aln->ft[i];
		n = aln->si[i]->ft;
	//	old_n = aln->ft[i];
		old_n = aln->si[i]->ft;
		while(n){
			old_n = n;
			while(old_n){
				//switch :
				//if(strlen(n->type) == strlen(old_n->type)){
				if(check_identity(n->type,old_n->type)!=-1){
				if(n->start > old_n->start){
					tmp = n->start;
					n->start = old_n->start;
					old_n->start = tmp;

					tmp = n->end;
					n->end = old_n->end;
					old_n->end = tmp;

					/*tmp = n->color;
					n->color = old_n->color;
					old_n->color = tmp;

					t = n->type;
					n->type = old_n->type;
					old_n->type = t;*/

					t = n->note;
					n->note = old_n->note;
					old_n->note = t;

				}
				}
			//	}
				old_n = old_n->next;
			}
			n = n->next;
		}
	}
	return aln;
}


void feature_coverage(struct alignment* aln,char* type)
{
	struct feature* n = 0;
	int i,j;


	float feature_residues = 0.0;
	float num_residues = 0.0;
	for (i = 0;i < aln->numseq;i++){
		for (j = 0; j < aln->aln_len;j++){
			if(isalpha((int)aln->si[i]->s[j])){
				num_residues+= 1;
			}
		}
	}
	
	for (i = 0;i < aln->numseq;i++){
		n = aln->si[i]->ft;
		while(n){
			if(check_identity(n->type,type)!=-1){
				if(n->rel_start >= 0 && n->rel_end >= 0){
					feature_residues +=  n->end - n->start+1;
				}
			}
			n = n->next;
		}
	}
	fprintf(stdout,"%s: %f\n",type,feature_residues/num_residues);
}

struct alignment* assign_color_codes(struct alignment* aln)
{
	struct utype_ufeat* utf = 0;
	struct feature* n = 0;
	struct feature* old_n = 0;
	struct feature* p = 0;
	int* counter = 0;
	int i,j;
	int tmp;
	char* t;

	utf = get_unique_features(aln,utf);
	
	n = utf->t;
	while(n){
		//fprintf(stderr,"%s\n",n->type);
		aln->numtypes++;
		n = n->next;
	}
	aln->types = tmalloc(sizeof(char*)*aln->numtypes);
	
	n = utf->t;
	i = 0;
	while(n){
		aln->types[i] = tmalloc(sizeof(char)*(strlen(n->type)+1));
		for (j=0;j < strlen(n->type);j++){
			aln->types[i][j] = n->type[j];
		}
		aln->types[i][j] = 0;
		n = n->next;
		i++;
	}


	counter =tmalloc(sizeof(int)*aln->numtypes);
	for (i = aln->numtypes;i--;){
		counter[i] = 1;
	}
	//sort features according to residues covered...
	n = utf->f;
	old_n = utf->f;
	while(n){
		old_n = n;
		while(old_n){

			if(n->start < old_n->start){
				tmp = n->start;
				n->start = old_n->start;
				old_n->start = tmp;
				
				tmp = n->end;
				n->end = old_n->end;
				old_n->end = tmp;


				t = n->note;
				n->note = old_n->note;
				old_n->note = t;
				
				t = n->type;
				n->type = old_n->type;
				old_n->type = t;
				

			}
			old_n = old_n->next;
		}
		n = n->next;
	}
	
	
	/*while(n){
	
	
		n = n->next;
	}*/
	
	
	
	// assign colors to features...
	n = utf->f;
	while(n){
		for (i = aln->numtypes;i--;){
			//fprintf(stderr,"Match:\n%s\n%s\n",aln->types[i],n->type);	
			//if(strlen(aln->types[i]) == strlen(n->type)){
			if(check_identity(aln->types[i],n->type)!= -1){
			//	fprintf(stderr,"YES\n");
				
				n->color = counter[i];
				//fprintf(stderr,"Match:%s and %s:%s color:%d\n",aln->types[i],n->type,n->note,n->color);	
				counter[i]++;
				break;
			}
			//}
			//fprintf(stderr,"\n");
		}
		n = n->next;
	}
	free(counter);
	//exit(0);
	for (i = aln->numseq;i--;){
	//	n = aln->ft[i];
		n = aln->si[i]->ft;
		while(n){
			p = utf->f;
			while(p){
				
				//if(strlen(n->note) == strlen(p->note)){
				if(check_identity(n->note,p->note)!= -1){
					n->color = p->color;
					break;
				}
				//}
				p = p->next;
			}
			n = n->next;
		}
	}
	free_utf(utf);
	return aln;
}

struct utype_ufeat* get_unique_features(struct alignment* aln,struct utype_ufeat* utf)
{
	int i;
	utf = tmalloc(sizeof(struct utype_ufeat)*1);
	utf->t = 0;
	utf->f = 0;
	//utf->types= 0;
	//utf->numtypes = 0;
	for (i =0; i < aln->numseq;i++){
	//	utf = traverse_ft(utf,aln->ft[i]);
		utf = traverse_ft(utf,aln->si[i]->ft);
	}
	return utf;
}


struct utype_ufeat* traverse_ft(struct utype_ufeat* utf,struct feature* n)
{
	if (n != NULL){
		//if(!n->color){
			utf->t = add_unique_type(utf->t,n);
			utf->f = add_unique_feature(utf->f,n);
		//}
		traverse_ft(utf,n->next);
	}
	return utf;
	//return color_code;
}

struct feature* add_unique_feature(struct feature *n, struct feature *toadd)
{
	int i;
		
        if (n == NULL){
		n = (struct feature*) tmalloc(sizeof(struct feature));
		n->type = tmalloc(sizeof(char)* (strlen(toadd->type)+1));
		for ( i= 0;i < strlen(toadd->type);i++){
			n->type[i] = toadd->type[i];
		}
		n->type[i] = 0;
		
		n->note = tmalloc(sizeof(char)* (strlen(toadd->note)+1));
		for ( i= 0;i < strlen(toadd->note);i++){
			n->note[i] = toadd->note[i];
		}
		n->note[i] = 0;
		
		n->start = toadd->end - toadd->start;
		n->end = 0;
		
		n->next = 0;
	}else{
		//check_identity(toadd->note,n->note) == -1
	
		//if(strlen(toadd->note) != strlen(n->note)){
		//	n->next = add_unique_feature(n->next,toadd);
		//}else{
	
			if((check_identity(toadd->note,n->note)== -1)){
				n->next = add_unique_feature(n->next,toadd);
			}else{
				n->start += toadd->end - toadd->start;
			}
		//}
	}
        return n;
}

struct feature* add_unique_type(struct feature *n, struct feature *toadd)
{
	int i;
		
        if (n == NULL){
        	//printf("NEW eNTRY:query:%s\n",type);
		n = (struct feature*) tmalloc(sizeof(struct feature));
		n->type = tmalloc(sizeof(char)* (strlen(toadd->type)+1));
		for ( i= 0;i < strlen(toadd->type);i++){
			n->type[i] = toadd->type[i];
		}
		n->type[i] = 0;
		
		n->note = tmalloc(sizeof(char)* (strlen(toadd->note)+1));
		for ( i= 0;i < strlen(toadd->note);i++){
			n->note[i] = toadd->note[i];
		}
		n->note[i] = 0;
		
		n->start = 0;
		n->end = 0;
		
		n->next = 0;
	}else{
		//if(strlen(toadd->type) != strlen(n->type)){
		//n->next = add_unique_type(n->next,toadd);
		//}else{
		if((check_identity(toadd->type,n->type)== -1)){
			n->next = add_unique_type(n->next,toadd);
		}
		//}
	}
        return n;
}

void print_ft(struct feature* n)
{
	//fprintf(stdout,"GAGA\n");
	 if (n != NULL){
	 	fprintf(stdout,"%s %d-%d (%s)	col:%d\n",n->type,n->start,n->end,n->note,n->color);
		print_ft(n->next);
	}
}

struct feature* find_free_node(struct feature *n)
{
        if (n == NULL){
		//n = (struct feature*) tmalloc(sizeof(struct feature));
		//n->next = 0;
		 return n;
		//n->pos = pos;
	}else{
		n->next = find_free_node(n->next);
	}
        return n;
}
