/*
	phylo_main.c 
	
	Released under GPL - see the 'COPYING' file   
	
	Copyright (C) 2006 Timo Lassmann <timolassmann@gmail.com>
	
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    
	Please send bug reports, comments etc. to:
	timolassmann@gmail.com
*/


#include "msa_viewer.h"


#include "phylo.h"
#include "svg_tree.h"



struct alignment* phylo (struct alignment* aln,char* in, char* outfile,char* tree_param)
{

	int node_limit = 100000;
	float** dm = 0;
	struct tnode* tree = 0;
	
	int i,j;
	if(aln){
		numseq = aln->numseq;
	
		dm = malloc(sizeof(float*)*numseq);
		for (i = 0; i < aln->numseq;i++){
			dm[i] = malloc(sizeof(float)*numseq);
			for (j = 0; j < numseq;j++){
				dm[i][j] = 0.0;
			}
		}
		dm = get_distances_from_multiple_alignment(aln,dm);
	
		for (i = 0; i < numseq;i++){
			for (j = 0; j < numseq;j++){
				dm[i][j] = 1- dm[i][j];
				if(dm[i][j] == 0.0){
					dm[i][j] = 0.01;
				}
				//fprintf(stderr,"%f ",dm[i][j]);
			}
			//fprintf(stderr,"\n");
		}
		//fprintf(stderr,"\n");
		//if(byg_start("NJ",tree_param)!= -1) {
		//	tree = dnj(dm,aln);
		//}else{
			tree = dupgma(dm,aln);
		//	tree = dnj(dm,aln);
		//}
	}else{
		tree = read_phylip_tree(tree,in);
	}
	if(numseq < node_limit){
	
	//tree = dnj(dm,aln);
	//print_ttree(tree);
	if(byg_start("cir",tree_param)!= -1){
		circular_svg_tree(tree);
		//fprintf(stderr,"Circular\n");
	}else{
		radial_svg_tree(tree);
		//fprintf(stderr,"Radial\n");
	}
	//fprintf(stderr,"%s\n",tree_param);
	
	svg_tree_out(tree,outfile);
	//print_ttree(tree);
	}else{
		fprintf(stderr,"<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"  ?>\n\
	<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.1//EN\" \"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd\">\n\
		<!-- Created with Kalignvu (http://msa.cgb.ki.se/) -->\n\
		<svg xmlns=\"http://www.w3.org/2000/svg\" width=\"100%%\" height=\"100%%\">");//,(int)((maxx -
		fprintf(stderr,"<text font-family = \"sans\" font-size =\"12\" x=\"50\" y=\"50\"> Currently tree drawing is only supported for trees with fewer than %d leaves.</text>",node_limit);  
		fprintf(stderr,"<text font-family = \"sans\" font-size =\"12\" x=\"50\" y=\"70\"> Your tree has %d leaves.</text>",numseq);  
		fprintf(stderr,"</svg>");
		//fprintf(stderr,"ERROR: Currently tree drawing is only possible for trees with fewer than %d nodes.\n",node_limit);  
	}
	
	free_ttree(tree);
	if(aln){
		for (i = 0; i < numseq;i++){
			free(dm[i]);
		}
		free(dm);
	}
	return aln;
}

float**  get_distances_from_multiple_alignment(struct alignment* aln,float** dm)
{
	int i,j,c;
	float id = 0.0;
	float positions = 0.0;
		
	for (i = 0; i< numseq;i++){
		for (j = i+1;j < numseq;j++){
			id = 0.0;
			positions  = 0.0;
			for (c = 0;c < aln->aln_len;c++){
				if (isalpha(aln->si[i]->s[c])){
					if (isalpha(aln->si[j]->s[c])){
						if (aln->si[i]->s[c] == aln->si[j]->s[c]){
							id += 1.0;
						}
						positions += 1.0;
					}
				}
			}
			if(positions){
				dm[i][j] = id/positions;
				dm[j][i] = id/positions;
				//positions = (aln->si[i]->sl + aln->si[j]->sl) /2.0;
				//dm[i][j] = id/positions;
				//dm[j][i] = id/positions;
				
			}else{
				dm[i][j] = 0.0;
				dm[j][i] = 0.0;
			}
			//fprintf(stderr,"%f	%s	%s	\n",dm[i][j],aln->si[i]->name,aln->si[j]->name);
		}
	}
	return dm;
}



struct tnode* dupgma(float **dm,struct alignment* aln)
{
	int i,j;

	float min = 0.0;
	int node_a = 0;
	int node_b = 0;
	int cnode = numseq;
	
	struct tnode** tree = 0;
	struct tnode* tmp = 0;
	
	
	tree = malloc(sizeof(struct tnode*)*numseq);
	for (i = 0;i < numseq;i++){
		tree[i] = alloc_node(tree[i],3);
		tree[i]->label = malloc(sizeof(char)*(aln->si[i]->namel+1));
		tree[i]->labell = aln->si[i]->namel+1;
		for (j = 0 ; j < tree[i]->labell;j++){
			tree[i]->label[j] = aln->si[i]->name[j];
		}
		tree[i]->label[tree[i]->labell-1] = 0;
		tree[i]->leaf = 1;
		tree[i]->size = 1;
		tree[i]->num = i;
	}
	
	while (cnode > 1){
		min = FLOATINFTY;
		for (i = 0;i < numseq-1; i++){
			if (tree[i]){
			for ( j = i + 1;j < numseq;j++){
				if (tree[j]){
				if (dm[i][j] < min){
					min = dm[i][j];
					node_a = i;
					node_b = j;
				}
				}
			}
			}
		}
		
		tmp = alloc_node(tmp,3);
		
		if(tree[node_a]->size > tree[node_b]->size){
			tmp->left = tree[node_a];
			tmp->right = tree[node_b];
			
			tmp->links[0] = tree[node_a];
			tmp->links[1] = tree[node_b];
		}else{
			tmp->left = tree[node_b];
			tmp->right = tree[node_a];
			
			tmp->links[1] = tree[node_a];
			tmp->links[0] = tree[node_b];
			
		}
		
		
		tmp->left->parent = tmp;
		tmp->right->parent = tmp;
		
		tmp->left->e = dm[node_a][node_b] / 2 - tmp->left->height;
		tmp->right->e = dm[node_a][node_b] / 2- tmp->right->height;
		tmp->left->deg +=1;
		tmp->right->deg +=1;
		
		
		tmp->deg = 2;
		
		tmp->height = dm[node_a][node_b]/2;
		
		tmp->size = tree[node_a]->size + tree[node_b]->size;
		tmp->depth = 1;
		if(tree[node_a]->depth > tree[node_b]->depth){
			tmp->depth += tree[node_a]->depth;
		}else{
			tmp->depth += tree[node_b]->depth;
		}
		tmp->num = numseq + numseq - cnode;
		
		
		tree[node_a] = tmp;
		tree[node_b] = 0;
				
		cnode--;    
		
		/*calculate new distances*/
		for (j = numseq;j--;){
			if (j != node_b){
				dm[node_a][j] = (dm[node_a][j] + dm[node_b][j])/2;
			}
		}
		dm[node_a][node_a] = 0.0f;
		for (j = numseq;j--;){
			dm[j][node_a] = dm[node_a][j];
			dm[j][node_b] = 0.0f;
			dm[node_b][j] = 0.0f;
		}		
	}
	tmp = tree[node_a];
	tree[node_a]->leaf = -1;
	tree[node_a]->e = 0.0;

	free(tree);
	return tmp;
}


struct tnode* dnj(float **dm,struct alignment* aln)
{
	int i,j;

	float min = 0.0;
	float* r = 0;
	int node_a = 0;
	int node_b = 0;
	int cnode = numseq;
	
	
	float** M = 0;
	
	M = malloc(sizeof(float*)*numseq);
	for (i = 0; i <numseq;i++){
		M[i] = malloc(sizeof(float)*numseq);
		for (j = 0; j < numseq;j++){
			M[i][j] = 0.0;
		}
		
	}
	
	
	
	
	struct tnode** tree = 0;
	struct tnode* tmp = 0;
	
	r = malloc(sizeof(float)*numseq);
	tree = malloc(sizeof(struct tnode*)*numseq);
	for (i = 0;i < numseq;i++){
		r[i] = 0.0;
		tree[i] = malloc(sizeof(struct tnode));
		tree[i]->left = 0;
		tree[i]->right = 0;
		tree[i]->parent = 0;
		tree[i]->label = malloc(sizeof(char)*aln->si[i]->namel);
		tree[i]->labell = aln->si[i]->namel;
		for (j = 0 ; j < tree[i]->labell;j++){
			tree[i]->label[j] = aln->si[i]->name[j];
		}
		tree[i]->label[tree[i]->labell-1] = 0;
		
		tree[i]->height = 0.0;
		tree[i]->x = 0.0;
		tree[i]->y = 0.0;
		tree[i]->w = 0.0;
		tree[i]->tau = 0.0;
		
		tree[i]->leaf = 1;
		tree[i]->size = 1;
		tree[i]->deg  = 0;
		tree[i]->depth = 0;
		tree[i]->num = i;
	}
	
	while (cnode > 1){
		min = FLOATINFTY;
		for (i = 0;i < numseq; i++){
			r[i] = 0.0;
		}

		for (i = 0;i < numseq-1; i++){
			if (tree[i]){
			for ( j = i + 1;j < numseq;j++){
				if (tree[j]){
					r[i] += dm[i][j];
					r[j] += dm[i][j];
				}
			}
			}
		}
		for (i = 0;i < numseq-1; i++){
			if (tree[i]){
			for ( j = i + 1;j < numseq;j++){
				if (tree[j]){
					if(cnode > 2){
						M[i][j] = dm[i][j] - (r[i] +r[j])/(cnode-2);
					}else{
						M[i][j]= dm[i][j];
					}
			//		fprintf(stderr,"%f	%f	%f	%f\n",dm[i][j],M[i][j],r[i],r[j]);
				//	M[j][i] = M[i][j];
				}
			}
			}
		}
		
		
		for (i = 0;i < numseq-1; i++){
			if (tree[i]){
			for ( j = i + 1;j < numseq;j++){
				if (tree[j]){
				if (M[i][j] < min){
					min = M[i][j];
					node_a = i;
					node_b = j;
				}
				}
			}
			}
		}
		
		//fprintf(stderr,"Picked: %d	%d\n",node_a,node_b);
		tmp = malloc(sizeof(struct tnode));
		if(tree[node_a]->size > tree[node_b]->size){
			tmp->left = tree[node_a];
			tmp->right = tree[node_b];
		}else{
			tmp->left = tree[node_b];
			tmp->right = tree[node_a];
		}
		tmp->left->parent = tmp;
		tmp->right->parent = tmp;

		tmp->label = 0;
		tmp->labell = 0;

		if( cnode > 2){
			tmp->left->e = (dm[node_a][node_b] / 2) + (r[node_a]-r[node_b]) / (2*(cnode-2));
			tmp->right->e = dm[node_a][node_b] - tmp->left->e;
		}else{
			tmp->left->e = (dm[node_a][node_b] / 2) ;
			tmp->right->e = dm[node_a][node_b] - tmp->left->e;
		}
		//fprintf(stderr,"%f	%f\n",tmp->left->e,tmp->right->e);
		
		tmp->height = 0.0;
		
		tmp->left->deg +=1;
		tmp->right->deg +=1;
		
		tmp->deg = 2;
		
		tmp->parent = 0;
		
		//tmp->height = dm[node_a][node_b] / 2;
		tmp->x = 0.0;
		tmp->y = 0.0;
		tmp->w = 0.0;
		tmp->tau = 0.0;
		
		
		tmp->leaf = 0;
		
		tmp->size =tree[node_a]->size +  tree[node_b]->size;
		tmp->depth = 1;
		if(tree[node_a]->depth > tree[node_b]->depth){
			tmp->depth += tree[node_a]->depth;
		}else{
			tmp->depth += tree[node_b]->depth;
		}
		tmp->num = numseq + numseq -cnode;
		
		tree[node_a] = tmp;
		tree[node_b] = 0;
				
		cnode--;
		
		
		for (j = 0;j < numseq;j++){
			if (j != node_b){
				dm[node_a][j] = (dm[node_a][j] + dm[node_b][j] - dm[node_a][node_b])/2;
			}
		}
		dm[node_a][node_a] = 0.0f;
		for (j = numseq;j--;){
			dm[j][node_a] = dm[node_a][j];
			dm[j][node_b] = 0.0f;
			dm[node_b][j] = 0.0f;
		}		
		
	}
	
	
	tmp = tree[node_a];
	tree[node_a]->leaf = -1;
	tree[node_a]->e = 0.0;
	
	for (i = 0; i <numseq;i++){
		free(M[i]);
	}
	free(M);
	
	free(r);
	free(tree);
	return tmp;
}

void print_ttree(struct tnode* p)
{
	int i = 0;
	
	while(p->links[i]){
		print_ttree(p->links[i]);
		i++;
	}
	
	//if(p->left){
	//	print_ttree(p->left);	
	//}
	
	fprintf(stderr,"Leaf:%d	",p->leaf);
	fprintf(stderr,"Deg:%d	",p->deg);
	fprintf(stderr,"height:%f	",p->height);
	
	fprintf(stderr,"size:%d	",p->size);
	fprintf(stderr,"depth:%d	",p->depth);
	fprintf(stderr,"x,y:%f	",p->x);
	fprintf(stderr,",%f	",p->y);
	if(p->label){
		fprintf(stderr,"num:%d	",p->num);
		fprintf(stderr,"%s	e:%f\n",p->label,p->e);
	}else{
		fprintf(stderr,"num:%d	",p->num);
		fprintf(stderr,"		e:%f\n",p->e);
	}

	//if(p->leaf != -1){
		//fprintf(stderr,"PArent is%d\n",p->parent->num);
	//}
	
	//if(p->right){
	//	print_ttree(p->right);	
	//}
	
}





