/*
	print_svg.c 
	
	Released under GPL - see the 'COPYING' file   
	
	Copyright (C) 2006 Timo Lassmann <timolassmann@gmail.com>
	
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    
	Please send bug reports, comments etc. to:
	timolassmann@gmail.com
*/

#include "msa_viewer.h"
#include "phylo.h"
#include "svg_tree.h"
#include <math.h>

double dist;
double scale;
double x_offset;
double y_offset;
double margin = 200;

double max_name_len = 0.0;

void svg_tree_out(struct tnode* p,char* outfile)
{
	//x_offset = minx;
	//y_offset = miny;
	
	dist = 3.0/(2*tan(M_PI /numseq));
	
	scale = 400/(maxx - minx); 
	
	if(minx < 0){
		x_offset = minx * -1;
	}else{
		x_offset = minx;
	}
	if(miny < 0){
		y_offset = miny * -1;
	}else{
		y_offset = miny;
	}
	
	find_margin(p);
	
	margin = max_name_len * 6 + dist+10;
	
	scale_svg_tree(p);
	
	FILE *fout = NULL;
		
	if(outfile){
		if ((fout = fopen(outfile, "w")) == NULL){
			fprintf(stderr,"can't open output\n");
			exit(0);
		}
	}else{
		fout = stdout; 
	}
	
	//fprintf(stderr,"Content-type: text/xml\n\n");

	fprintf(fout,"<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"  ?>\n\
	<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.1//EN\" \"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd\">\n\
		<!-- Created with Kalignvu (http://msa.cgb.ki.se/) -->\n\
		<svg xmlns=\"http://www.w3.org/2000/svg\" width=\"100%%\" height=\"100%%\">");//,(int)((maxx - minx)*scale+2*margin),(int)((maxy - miny)*scale+2*margin));
	
	fprintf(fout,"<rect x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\" style=\"fill:blue;stroke:black;stroke-width:2;fill-opacity:0.0;\"/>",0,0,(int)((maxx - minx)*scale+2*margin),(int)((maxy - miny)*scale+2*margin));
	
	if(sigma){
		fprintf(fout,"<circle cx=\"%f\" cy=\"%f\" r=\"%f\" style=\"fill:rgb(%d,%d,%d)\"/>",x_offset*scale+margin,y_offset*scale+margin,scale,230,230,230);
	}
	//fprintf(stderr,"<svg:svg width=\"%fpx\" height=\"%fpx\">",maxx-minx*scale,maxy-miny*scale);
	

	//fprintf(stderr,"MINX:%f\nMAXX:%f\nMINY:%f\nMAXY:%f\n",minx,maxx,miny,maxy);
	print_svg_tree(p,fout);
	fprintf(fout,"</svg>");
	if(outfile){
		fclose(fout);
	}
}


void find_margin(struct tnode* p)
{
	int i = 0;
	
	
	while(p->links[i]){
		find_margin(p->links[i]);
		i++;
	}
	
	if(p->labell > max_name_len){
		max_name_len = p->labell;
	}
}

void scale_svg_tree(struct tnode* p)
{
	int i = 0;
	
	
	while(p->links[i]){
		scale_svg_tree(p->links[i]);
		i++;
	}
	
	
	if(p->labell > max_name_len){
		max_name_len = p->labell;
	}
	
	/*if(p->left){
		scale_svg_tree(p->left);	
	}
	if(p->right){
		scale_svg_tree(p->right);	
	}*/
	//fprintf(stderr,"before(%f,%f)	off:(%f,%f)\n",p->x,p->y,x_offset,y_offset);
	p->x = ((x_offset + p->x)*scale)+margin;
	p->y = ((y_offset + p->y)*scale)+margin;
	//fprintf(stderr,"after(%f,%f)\n",p->x,p->y);
	
}

void print_svg_tree(struct tnode* p,FILE* fout)
{
	double textx = 0.0;
	double texty = 0.0;
	double angle = 0.0;

	//double dist = 0.0;
	double fe = 0.0;
	
	int r = 99;
	int g = 99;
	int b = 99;
	
	int i = 0;
	
	
	while(p->links[i]){
		print_svg_tree(p->links[i],fout);	
		i++;
	}
	//if(p->left){
	//	print_svg_tree(p->left,fout,aln);	
	//}
	
	//fprintf(stderr,"%f	%f	\n",p->x,minx);
	
	//fprintf(stderr,"%f\n",sigma);
	if(p->parent){
		if(sigma){
			r = 0;
			g = 0;
			b = 0;
			fe = sqrt(pow(p->parent->x - p->x,2) + pow(p->parent->y - p->y,2)) / (sigma*p->e*scale);
			//fprintf(stderr,"fe:%f\n",fe);
			
			if(fe < 0.5){
				b = 255;
			}else if(fe < 1.0){
				b = (int)(-log(fe)  * 255.0);
			}else if(fe < 2.0){
				r = (int)(log(fe) * 255.0);
			}else{
				r = 255;
			}
		}
		fprintf(fout,"<line x1=\"%d\" y1=\"%d\" x2=\"%d\" y2=\"%d\" style=\"stroke:rgb(%d,%d,%d);stroke-width:2\"/>\n",(int)p->x,(int)p->y,(int)p->parent->x,(int)p->parent->y,r,g,b);
		
	}

	if(p->leaf == 1){	
		fprintf(fout,"<circle cx=\"%f\" cy=\"%f\" r=\"3\" stroke=\"black\" stroke-width=\"2\" 	fill=\"red\"/>\n",p->x,p->y);

		

		if(cos(p->tau+ p->w/2) > 0){
			textx = p->x + dist * cos(p->tau+ p->w/2);
			texty = p->y+ dist * sin(p->tau+ p->w/2);
			angle = (p->tau + p->w/2.0)*360.0/(2.0*M_PI);
		}else{
			//textx = p->x + (dist+aln->si[p->num]->namel * 7) *cos(p->tau+ p->w/2);
			//texty = p->y+ (dist+aln->si[p->num]->namel * 7) *sin(p->tau+ p->w/2);
			textx = p->x + (dist + p->labell * 6) *cos(p->tau+ p->w/2);
			texty = p->y+ (dist + p->labell * 6) *sin(p->tau+ p->w/2);
			angle = (p->tau+ p->w/2.0)*360.0/(2.0*M_PI)-180.0;
		}
		//fprintf(fout,"<text font-family = \"helvetica\" font-size =\"12\" x=\"%d\" y=\"%d\" transform = \"rotate(%d %d %d)\">%s</text>",(int)textx,(int)texty,(int)angle,(int)textx,(int)texty,aln->si[p->num]->name);
		fprintf(fout,"<text font-family = \"monospace\" font-size =\"12\" x=\"%d\" y=\"%d\" transform = \"rotate(%d %d %d)\">%s</text>",(int)textx,(int)texty,(int)angle,(int)textx,(int)texty,p->label);

	}//else if(p->leaf == -1){
	//	fprintf(fout,"<circle cx=\"%f\" cy=\"%f\" r=\"4\" stroke=\"black\" stroke-width=\"2\" 	fill=\"green\"/>\n",p->x,p->y);
	//}//else{
	//	fprintf(fout,"<circle cx=\"%f\" cy=\"%f\" r=\"4\" stroke=\"black\" stroke-width=\"2\" 	fill=\"red\"/>\n",p->x,p->y);
	//}

	
	//if(p->right){
	//	print_svg_tree(p->right,fout,aln);	
	//}
	
}



