/*
	read_tree.c 
	
	Released under GPL - see the 'COPYING' file   
	
	Copyright (C) 2006 Timo Lassmann <timolassmann@gmail.com>
	
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    
	Please send bug reports, comments etc. to:
	timolassmann@gmail.com
*/

#include "msa_viewer.h"
#include "phylo.h"
#include <string.h>

void bubble_sort_nodes(struct tnode* p);
struct tnode* make_tree(struct tnode* tp,struct tnode* parent,char* string);
void clean_tree(struct tnode* p);

int intern = 0;

 struct tnode* read_phylip_tree(struct tnode* tree,char* string)
 {
 	
 	tree = make_tree(tree,0,string);
	tree->leaf = -1;
	
	
	clean_tree(tree);

	free(string);
 	return tree;
 }
 
void clean_tree(struct tnode* p)
{
	int i = 0;
		
	while(p->links[i]){
		clean_tree(p->links[i]);
		i++;
	}
	
	//if(p->left){
	//	clean_tree(p->left);
	//}
	if(p->leaf != 1){
		i = 0;
		p->size = 0;
		while(p->links[i]){
			p->size +=  p->links[i]->size;
			i++;
		}
		//p->size = p->left->size + p->right->size;
		p->num = numseq+intern;
		intern++;
		
		
		i = 0;
		p->depth = -1;
		p->height = 0;
		while(p->links[i]){
			if(p->links[i]->depth > p->depth){
				p->depth = p->links[i]->depth;
			}
			if(p->links[i]->height+p->links[i]->e > p->height){
				 p->height = p->links[i]->height+p->links[i]->e;
			}
			i++;
		}
		p->depth++;
		
		/*if(p->left->depth > p->right->depth){
			p->depth += p->left->depth;
		
		}else{
			p->depth += p->right->depth;
		}*/
	}
	
	bubble_sort_nodes(p);
	//if(p->right){
	//	clean_tree(p->right);
	//}
}

void bubble_sort_nodes(struct tnode* p)
{
	struct tnode* tmp = 0;
	register unsigned int i,j;
	register unsigned int swap = 0;
	int array_size = 0;	
	
	while(p->links[array_size]){
		array_size++;
	}
	//fprintf(stderr,"array_size:%d	\n",array_size);
	for (i = 0; i < array_size;i++){
		swap = 0;
		for(j = 0; j < array_size-1;j++){		
			if((p->links[j]->e+p->links[j]->height) > (p->links[j+1]->e + p->links[j+1]->height) ){
				
				//fprintf(stderr,"%f	%f	%f	%f	%p	%p\n",(p->links[j]->e+p->links[j]->height),(p->links[j+1]->e + p->links[j+1]->height),p->links[j]->height,p->links[j+1]->height,p->links[j], p->links[j+1]);
				tmp = p->links[j];
				p->links[j] = p->links[j+1];
				p->links[j+1] =  tmp;
				swap = 1;
				//fprintf(stderr,"%f	%f	%p	%p\n",(p->links[j]->e+p->links[j]->height),(p->links[j+1]->e + p->links[j+1]->height),p->links[j], p->links[j+1]);
			}		
		}
		if(!swap){
			i = array_size;
		}
	}
}

 
struct tnode* make_tree(struct tnode* tp,struct tnode* parent,char* string)
 {
 	float e = 0.0;
 	char*p = string;
 	int open = 0;
  	//int mid1 = -1;
  	//int mid2 = -1;
  	int i,j,c,m;
  	
  	int degree = byg_count(",",p);
  	int* mid = 0;
  	
  	
	//fprintf(stderr,"input:\n%s - %d\n",p,degree);
  	
  	m = 0;
 	if((i = byg_start("(",p))!= -1){
 		mid = malloc(sizeof(int)*(degree +1));
 		for (j = 0;j < degree+1;j++){
 			mid[j] = -1;
 		}
 		p+= i; 
		open  = 0;
		i = 0;
		while(p[i]){
			c = (int)p[i];
			
			if(c == '('){
				open++;
			}else if(c ==','){
				if (open == 1){
					while(mid[m] != -1){
						m++;
					}
					mid[m] = i+1;
					/*if(mid1 != -1 ){
						fprintf(stderr,"NO BINARY TREE:%s\n",p);
						//exit(-1);
						mid2 = i+1;
					}else{
						mid1 = i+1;
					}*/
					p[i] = 0;
					//make_tree(tp,p+1);
					//p+=i+1;
					//i = 0;
				}
			
			}else if(c == ')'){
				open --;
				if(!open){	
					p[i] = 0;
					i++;
					while(p[i]){
						if(p[i] ==':'){
							while(!isalnum((int)p[i])){ 
								i++;
							}
						
							e = atof(p+i);
						}
						i++;
					}
				
					p[i] = 0;
					break;
				}
			}
			//fprintf(stderr,"%c	%d\n",p[i],open);
			i++;
		}
		tp = alloc_node(tp,m+3);
 		if(parent){
 			tp->parent = parent;
 			tp->deg = m+3;
 		}else{
 			tp->deg = m+2;
 		}
 		if(!e){
 			e = 0.01;
 		}
 		tp->e = e;
		
		
		/*fprintf(stderr,"out:%d	%d	%d	\n%s\n",m+2,mid1,mid2,p+1);
		if(mid1 != -1){
			fprintf(stderr,"MID1:%s\n",p+mid1);
		}
		if(mid2 != -1){
			fprintf(stderr,"MID2:%s\n",p+mid2);
		}*/
		tp->links[0] = make_tree(tp->links[0],tp,p+1);
		
		for (c = 0; c < m+1;c++){
			//fprintf(stderr,"MIDC:%d ",mid[c]);
			if(mid[c] != -1){
			tp->links[c+1] = make_tree(tp->links[c+1],tp,p+mid[c]);
			}
		}
		//fprintf(stderr,"\n");
		
		free(mid);
		//fprintf(stderr,"%f\n",tp->e);
		//tp->left = make_tree(tp->left,tp,p+1);
		//tp->right = make_tree(tp->right,tp,p+mid1);
		//exit(0);
 	}else{
 		//fprintf(stderr,"LEAF???:%s\n",p);
 		tp = alloc_node(tp,3);
 		tp->num = numseq;
 		numseq++;
 		tp->deg = 1;
 		tp->size = 1;
		tp->depth = 0;
 		if(parent){
 			tp->parent = parent;
 		}
 		
 		tp->leaf = 1;
 		
 		j = 0;
 		while(!isalnum((int)p[j])){
 			j++;
 		}
 		p+=j;
 		
 		
 		if((j = byg_start(":",p)) != -1){
 			tp->label = malloc(sizeof(char)*(j+1));
 			tp->labell = j+1;
 			c = 0;
 			for (i = 0;i < (j + 1); i++){
 				if(isalnum((int)p[i])){
 					tp->label[c] = p[i];
 					c++;
 				}
 			//	fprintf(stderr,"%c",p[i]);
 			}
 			//fprintf(stderr,"\n");
 			tp->label[c] = 0;
 			//fprintf(stderr,"LABEL:>%s<\n",tp->label);
 			p+=j;
 			while(!isalnum((int)*p)){
 				p++;
 			}
 			tp->e = atof(p);
 			
 		}else{
 			j = strlen(p);
 			tp->label = malloc(sizeof(char)*j);
 			tp->labell = j;
 			c = 0;
 			for (i = 0;i < j ; i++){
 				if(isalnum((int)p[i])){
 					tp->label[c] = p[i];
 					c++;
 				}
 			//	fprintf(stderr,"%c",p[i]);
 			}
 			//fprintf(stderr,"\n");
 			tp->label[c] = 0;
 			tp->e = 0.01;
 			//fprintf(stderr,"LABEL:%s\n",tp->label);
 		}
 	}
 	return tp;
}

